@@ -4,7 +4,7 @@
  *
  * @section LICENSE
  *
- *      Copyright 2009, 2010, 2011 <qmidiarp-devel@lists.sourceforge.net>
+ *      Copyright 2009, 2010, 2011, 2012 <qmidiarp-devel@lists.sourceforge.net>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -57,9 +57,9 @@ class ArpScreen : public QWidget
   Q_OBJECT
 
   private:
+    QTimer *timer;
     int grooveTick, grooveVelocity, grooveLength;
     QString pattern;
-    QString a_pattern;
     int pattern_updated, currentIndex;
     bool isMuted;
 
@@ -71,9 +71,16 @@ class ArpScreen : public QWidget
     ~ArpScreen();
     virtual QSize sizeHint() const;
     virtual QSizePolicy sizePolicy() const;
+    int maxOctave;
+    int minOctave;
+    double minStepWidth;
+    double nSteps;
+    int patternMaxIndex;
 
   public slots:
-    void updateScreen(const QString&);
+    void updateScreen(const QString& pattern, int p_minOct, int p_maxOct,
+                            double p_minStepWidth, double p_nSteps,
+                            int p_patternMaxIndex);
     void updateScreen(int p_index);
     void newGrooveValues(int tick, int vel, int length);
     void setMuted(bool on);
