@@ -4,7 +4,7 @@
  *
  * @section LICENSE
  *
- *      Copyright 2009, 2010, 2011 <qmidiarp-devel@lists.sourceforge.net>
+ *      Copyright 2009, 2010, 2011, 2012 <qmidiarp-devel@lists.sourceforge.net>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -23,7 +23,6 @@
  */
 
 #include <QBoxLayout>
-#include <QStringList>
 #include <QFile>
 #include <QTextStream>
 #include <QInputDialog>
@@ -48,9 +47,10 @@ ArpWidget::ArpWidget(MidiArp *p_midiWorker, int portCount, bool compactStyle,
     QWidget(parent), midiWorker(p_midiWorker), modified(false)
 {
     int l1;
-    QStringList midiCCNames;
-    midiCCNames << "MuteToggle" << "PresetSwitch" << "unknown";
-    midiControl = new MidiControl(midiCCNames);
+
+    parStore = new ParStore();
+
+    midiControl = new MidiControl(this);
 
     manageBox = new ManageBox("Arp:", false, this);
 
@@ -127,7 +127,7 @@ ArpWidget::ArpWidget(MidiArp *p_midiWorker, int portCount, bool compactStyle,
     muteOut = new QCheckBox(this);
     connect(muteOut, SIGNAL(toggled(bool)), this, SLOT(setMuted(bool)));
     muteLabel->setBuddy(muteOut);
-    midiControl->addMidiLearnMenu(muteOut, 0);
+    midiControl->addMidiLearnMenu("MuteToggle", muteOut, 0);
 
     QLabel *portLabel = new QLabel(tr("&Port"), portBox);
     portOut = new QComboBox(portBox);
@@ -199,7 +199,7 @@ ArpWidget::ArpWidget(MidiArp *p_midiWorker, int portCount, bool compactStyle,
     patternPresetBox->setMinimumContentsLength(20);
     connect(patternPresetBox, SIGNAL(activated(int)), this,
             SLOT(selectPatternPreset(int)));
-    midiControl->addMidiLearnMenu(patternPresetBox, 1);
+    midiControl->addMidiLearnMenu("PresetSwitch", patternPresetBox, 1);
 
     repeatPatternThroughChord = new QComboBox(patternBox);
     QStringList repeatPatternNames;
@@ -212,7 +212,8 @@ ArpWidget::ArpWidget(MidiArp *p_midiWorker, int portCount, bool compactStyle,
 
     triggerMode = new QComboBox(patternBox);
     QStringList triggerModeNames;
-    triggerModeNames << tr("No trigger") << tr("Kbd restart") << tr("Kbd trigger");
+    triggerModeNames << tr("No trigger") << tr("Kbd restart") << tr("Kbd trigger")
+                        << tr("Kbd restart legato") << tr("Kbd trigger legato");
     triggerMode->insertItems(0, triggerModeNames);
     triggerMode->setToolTip(tr("Trigger Mode"));
     connect(triggerMode, SIGNAL(currentIndexChanged(int)), this,
@@ -454,6 +455,8 @@ void ArpWidget::writeData(QXmlStreamWriter& xml)
 
         midiControl->writeData(xml);
 
+        parStore->writeData(xml);
+
     xml.writeEndElement();
 }
 
@@ -553,28 +556,14 @@ void ArpWidget::readData(QXmlStreamReader& xml)
         else if (xml.isStartElement() && (xml.name() == "midiControllers")) {
             midiControl->readData(xml);
         }
+        else if (xml.isStartElement() && (xml.name() == "globalStores")) {
+            parStore->readData(xml);
+        }
         else skipXmlElement(xml);
     }
     modified = false;
 }
 
-void ArpWidget::skipXmlElement(QXmlStreamReader& xml)
-{
-    if (xml.isStartElement()) {
-        qWarning("Unknown Element in XML File: %s",qPrintable(xml.name().toString()));
-        while (!xml.atEnd()) {
-            xml.readNext();
-
-            if (xml.isEndElement())
-                break;
-
-            if (xml.isStartElement()) {
-                skipXmlElement(xml);
-            }
-        }
-    }
-}
-
 void ArpWidget::setChIn(int value)
 {
     chIn->setCurrentIndex(value);
@@ -598,8 +587,10 @@ void ArpWidget::updateText(const QString& newtext)
     patternPresetBox->setCurrentIndex(0);
     textRemoveAction->setEnabled(false);
     textStoreAction->setEnabled(true);
-    screen->updateScreen(newtext);
     midiWorker->updatePattern(newtext);
+    screen->updateScreen(newtext, midiWorker->minOctave,
+                            midiWorker->maxOctave, midiWorker->minStepWidth,
+                            midiWorker->nSteps, midiWorker->patternMaxIndex);
     modified = true;
 }
 
@@ -849,6 +840,54 @@ void ArpWidget::setModified(bool m)
     midiControl->setModified(m);
 }
 
+void ArpWidget::storeParams(int ix, bool empty)
+{
+    parStore->temp.empty = empty;
+    parStore->temp.muteOut = muteOut->isChecked();
+    parStore->temp.chIn = chIn->currentIndex();
+    parStore->temp.channelOut = channelOut->currentIndex();
+    parStore->temp.portOut = portOut->currentIndex();
+    parStore->temp.indexIn0 = indexIn[0]->value();
+    parStore->temp.indexIn1 = indexIn[1]->value();
+    parStore->temp.rangeIn0 = rangeIn[0]->value();
+    parStore->temp.rangeIn1 = rangeIn[1]->value();
+    parStore->temp.attack = attackTime->value();
+    parStore->temp.release = releaseTime->value();
+    parStore->temp.rndTick = randomTick->value();
+    parStore->temp.rndLen = randomLength->value();
+    parStore->temp.rndVel = randomVelocity->value();
+    parStore->temp.pattern = patternText->text();
+    parStore->temp.repeatMode = repeatPatternThroughChord->currentIndex();
+    parStore->tempToList(ix);
+}
+
+void ArpWidget::restoreParams(int ix)
+{
+    if (parStore->list.at(ix).empty) return;
+    patternText->setText(parStore->list.at(ix).pattern);
+    repeatPatternThroughChord->setCurrentIndex(parStore->list.at(ix).repeatMode);
+    updateRepeatPattern(parStore->list.at(ix).repeatMode);
+    indexIn[0]->setValue(parStore->list.at(ix).indexIn0);
+    indexIn[1]->setValue(parStore->list.at(ix).indexIn1);
+    rangeIn[0]->setValue(parStore->list.at(ix).rangeIn0);
+    rangeIn[1]->setValue(parStore->list.at(ix).rangeIn1);
+    attackTime->setValue(parStore->list.at(ix).attack);
+    releaseTime->setValue(parStore->list.at(ix).release);
+    randomTick->setValue(parStore->list.at(ix).rndTick);
+    randomLength->setValue(parStore->list.at(ix).rndLen);
+    randomVelocity->setValue(parStore->list.at(ix).rndVel);
+
+    muteOut->setChecked(parStore->list.at(ix).muteOut);
+    chIn->setCurrentIndex(parStore->list.at(ix).chIn);
+    updateChIn(parStore->list.at(ix).chIn);
+    channelOut->setCurrentIndex(parStore->list.at(ix).channelOut);
+    updateChannelOut(parStore->list.at(ix).channelOut);
+    setPortOut(parStore->list.at(ix).portOut);
+    updatePortOut(parStore->list.at(ix).portOut);
+
+    midiWorker->advancePatternIndex(true);
+}
+
 void ArpWidget::handleController(int ccnumber, int channel, int value)
 {
     bool m;
