@@ -4,7 +4,7 @@
  *
  * @section LICENSE
  *
- *      Copyright 2009, 2010, 2011 <qmidiarp-devel@lists.sourceforge.net>
+ *      Copyright 2009, 2010, 2011, 2012 <qmidiarp-devel@lists.sourceforge.net>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -29,9 +29,13 @@
 #include "engine.h"
 
 
-Engine::Engine(GrooveWidget *p_grooveWidget, int p_portCount, bool p_alsamidi, QWidget *parent) : QThread(parent), modified(false)
+Engine::Engine(GlobStore *p_globStore, GrooveWidget *p_grooveWidget, int p_portCount, bool p_alsamidi, QWidget *parent) : QThread(parent), modified(false)
 {
     ready = false;
+    globStoreWidget = p_globStore;
+    connect(globStoreWidget->midiControl, SIGNAL(setMidiLearn(int, int, int)),
+            this, SLOT(setMidiLearn(int, int, int)));
+
     grooveWidget = p_grooveWidget;
     connect(grooveWidget, SIGNAL(newGrooveTick(int)),
             this, SLOT(setGrooveTick(int)));
@@ -62,6 +66,23 @@ Engine::Engine(GrooveWidget *p_grooveWidget, int p_portCount, bool p_alsamidi, Q
     status = false;
     sendLogEvents = false;
     useMidiClock = false;
+    currentTick = 0;
+    requestTick = 0;
+
+    restoreRequest = -1;
+    restoreModIx = 0;
+    restoreModType = 'X';
+    restoreModWindowIndex = -1;
+    restoreTick = -1;
+
+    // we happen to need these QSignals for triggering display updates
+    // and global parameter restores from within the echo callback.
+    // Direct calls to the module widgets were shown to
+    // cause frequent segfaults.
+    connect(this, SIGNAL(restoreSig(int)), this, SLOT(restore(int)));
+    connect(this, SIGNAL(indicPercentSig(int)), this, SLOT(indicPercent(int)));
+    connect(this, SIGNAL(updateCursorSig(QChar, int, int))
+            , this, SLOT(updateCursor(QChar, int, int)));
 
     resetTicks(0);
     ready = true;
@@ -305,13 +326,15 @@ void Engine::updateIDs(int curID)
     int l1, tempDockID;
     for (l1 = 0; l1 < arpWidgetCount(); l1++) {
         arpWidget(l1)->manageBox->ID = l1;
+        arpWidget(l1)->setProperty("widgetID", l1);
         tempDockID = arpWidget(l1)->manageBox->parentDockID;
         if (tempDockID > curID) {
             arpWidget(l1)->manageBox->parentDockID = tempDockID - 1;
-            }
+        }
     }
     for (l1 = 0; l1 < lfoWidgetCount(); l1++) {
         lfoWidget(l1)->manageBox->ID = l1;
+        lfoWidget(l1)->setProperty("widgetID", l1);
         tempDockID = lfoWidget(l1)->manageBox->parentDockID;
         if (tempDockID > curID) {
             lfoWidget(l1)->manageBox->parentDockID = tempDockID - 1;
@@ -319,6 +342,7 @@ void Engine::updateIDs(int curID)
     }
     for (l1 = 0; l1 < seqWidgetCount(); l1++) {
         seqWidget(l1)->manageBox->ID = l1;
+        seqWidget(l1)->setProperty("widgetID", l1);
         tempDockID = seqWidget(l1)->manageBox->parentDockID;
         if (tempDockID > curID) {
             seqWidget(l1)->manageBox->parentDockID = tempDockID - 1;
@@ -425,9 +449,14 @@ void Engine::echoCallback(bool echo_from_trig)
     int note_tick = 0;
     int length;
     int outport;
+    int frameptr;
+    int percent;
     bool isNew;
+    bool restoreFlag = (restoreRequest >= 0);
     MidiEvent outEv;
 
+    currentTick = tick;
+
     note.clear();
     velocity.clear();
 
@@ -446,7 +475,8 @@ void Engine::echoCallback(bool echo_from_trig)
                     outEv.type = EV_CONTROLLER;
                     outEv.data = midiLfo(l1)->ccnumber;
                     outEv.channel = midiLfo(l1)->channelOut;
-                    lfoWidget(l1)->screen->updateScreen(lfoWidget(l1)->getFramePtr());
+                    frameptr = midiLfo(l1)->getFramePtr();
+                    emit updateCursorSig('L', l1, frameptr);
                     midiLfo(l1)->getNextFrame(&lfoData, tick);
                     outport = midiLfo(l1)->portOut;
                     if (!midiLfo(l1)->isMuted) {
@@ -460,6 +490,16 @@ void Engine::echoCallback(bool echo_from_trig)
                             l2++;
                         }
                     }
+                    if ((restoreModType == 'L') && (l1 == restoreModIx)
+                            && (!globStoreWidget->timeModeBox->currentIndex())) {
+                        frameptr = midiLfo(l1)->getFramePtr();
+                        percent = frameptr * 100 / (midiLfo(l1)->nPoints);
+                        emit indicPercentSig(percent);
+                        if (!frameptr && restoreFlag) {
+                            restoreTick = lfoData.last().tick;
+                            restoreFlag = false;
+                        }
+                    }
                 }
             }
             if (!l1)
@@ -480,7 +520,7 @@ void Engine::echoCallback(bool echo_from_trig)
                     outEv.type = EV_NOTEON;
                     outEv.value = midiSeq(l1)->vel;
                     outEv.channel = midiSeq(l1)->channelOut;
-                    seqWidget(l1)->screen->updateScreen(seqWidget(l1)->getCurrentIndex());
+                    emit updateCursorSig('S', l1, midiSeq(l1)->getCurrentIndex());
                     midiSeq(l1)->getNextNote(&seqSample, tick);
                     length = midiSeq(l1)->notelength;
                     outport = midiSeq(l1)->portOut;
@@ -488,6 +528,16 @@ void Engine::echoCallback(bool echo_from_trig)
                         outEv.data = seqSample.value;
                         driver->sendMidiEvent(outEv, seqSample.tick, outport, length);
                     }
+                    if ((restoreModType == 'S') && (l1 == restoreModIx)
+                          && (!globStoreWidget->timeModeBox->currentIndex())) {
+                        frameptr = midiSeq(l1)->getCurrentIndex();
+                        percent = frameptr * 100 / (midiSeq(l1)->nPoints);
+                        emit indicPercentSig(percent);
+                        if (!frameptr && restoreFlag) {
+                            restoreTick = midiSeq(l1)->nextTick;
+                            restoreFlag = false;
+                        }
+                    }
                 }
             }
             if (!l1)
@@ -514,7 +564,7 @@ void Engine::echoCallback(bool echo_from_trig)
                     length = midiArp(l1)->returnLength * 4;
                     outport = midiArp(l1)->portOut;
                     isNew = midiArp(l1)->returnIsNew;
-                    arpWidget(l1)->screen->updateScreen(arpWidget(l1)->getGrooveIndex());
+                    emit updateCursorSig('A', l1, midiArp(l1)->getGrooveIndex());
                     if (!velocity.isEmpty()) {
                         if (isNew && velocity.at(0)) {
                             l2 = 0;
@@ -526,6 +576,16 @@ void Engine::echoCallback(bool echo_from_trig)
                             }
                         }
                     }
+                    if ((restoreModType == 'A') && (l1 == restoreModIx)
+                            && (!globStoreWidget->timeModeBox->currentIndex())) {
+                        frameptr = midiArp(l1)->getGrooveIndex() - 1;
+                        percent = frameptr * 100 / (midiArp(l1)->nPoints);
+                        emit indicPercentSig(percent);
+                        if (!frameptr && restoreFlag) {
+                            restoreTick = note_tick;
+                            restoreFlag = false;
+                        }
+                    }
                 }
             }
             if (!l1)
@@ -537,6 +597,20 @@ void Engine::echoCallback(bool echo_from_trig)
         if (0 > nextMinArpTick) nextMinArpTick = 0;
         if (midiArpCount()) driver->requestEchoAt(nextMinArpTick, 0);
     }
+
+    if (restoreFlag && (globStoreWidget->timeModeBox->currentIndex())) {
+        percent = 100 * (currentTick - requestTick) / (restoreTick - requestTick);
+        emit indicPercentSig(percent);
+    }
+
+    if ((restoreTick > -1)
+        && (!midiArpCount() || (nextMinArpTick + schedDelayTicks >= restoreTick))
+        && (!midiLfoCount() || (nextMinLfoTick >= restoreTick))
+        && (!midiSeqCount() || (nextMinSeqTick >= restoreTick))) {
+        restoreTick = -1;
+        restoreFlag = false;
+        emit restoreSig(restoreRequest);
+    }
 }
 
 bool Engine::midi_event_received_callback(void * context, MidiEvent ev)
@@ -608,6 +682,7 @@ void Engine::sendController(int ccnumber, int channel, int value)
     int l1;
 
     grooveWidget->handleController(ccnumber, channel, value);
+    globStoreWidget->handleController(ccnumber, channel, value);
 
     for (l1 = 0; l1 < arpWidgetCount(); l1++)
         arpWidget(l1)->handleController(ccnumber, channel, value);
@@ -625,6 +700,14 @@ void Engine::learnController(int ccnumber, int channel)
         midiLearnFlag = false;
         return;
         }
+
+    if (midiLearnWindowID == -2) {
+        globStoreWidget->midiControl->appendMidiCC(midiLearnID,
+                ccnumber, channel, 0, 127);
+        midiLearnFlag = false;
+        return;
+        }
+
     int min = (midiLearnID) ? 0 : 127; //if control is toggle min=max
     if (moduleWindow(midiLearnWindowID)->objectName().startsWith("Arp")) {
         arpWidget(midiLearnModuleID)->midiControl->appendMidiCC(midiLearnID,
@@ -710,3 +793,113 @@ void Engine::tr_state_cb(bool on, void *context)
         }
     }
 }
+
+void Engine::indicPercent(int p)
+{
+    globStoreWidget->indicator->updatePercent(p);
+}
+
+void Engine::updateCursor(QChar modtype, int ix, int pos)
+{
+    switch (modtype.toLatin1()) {
+
+        case 'S':
+            seqWidget(ix)->screen->updateScreen(pos);
+        break;
+
+        case 'L':
+            lfoWidget(ix)->updateScreen(pos);
+        break;
+
+        case 'A':
+            arpWidget(ix)->screen->updateScreen(pos);
+        break;
+
+        default: ;
+    }
+}
+void Engine::globStore(int ix)
+{
+    int l1;
+    for (l1 = 0; l1 < arpWidgetCount(); l1++) {
+        arpWidget(l1)->storeParams(ix);
+    }
+    for (l1 = 0; l1 < lfoWidgetCount(); l1++) {
+        lfoWidget(l1)->storeParams(ix);
+    }
+    for (l1 = 0; l1 < seqWidgetCount(); l1++) {
+        seqWidget(l1)->storeParams(ix);
+    }
+    globStoreWidget->setDispState(ix, 1);
+}
+
+void Engine::requestRestore(int windowIndex, int ix)
+{
+    restoreModWindowIndex = windowIndex;
+
+    if (status == false) {
+        restore(ix);
+    }
+    else {
+        restoreRequest = ix;
+        globStoreWidget->setDispState(ix, 2, windowIndex);
+        if (globStoreWidget->timeModeBox->currentIndex()) {
+            requestTick = currentTick;
+            restoreTick = TPQN * (2 + globStoreWidget->switchAtBeatBox
+                ->currentIndex() + currentTick / TPQN);
+        }
+    }
+}
+
+void Engine::restore(int ix)
+{
+    if (restoreModWindowIndex < 0) {
+        int l1;
+        for (l1 = 0; l1 < arpWidgetCount(); l1++) {
+            arpWidget(l1)->restoreParams(ix);
+        }
+        for (l1 = 0; l1 < lfoWidgetCount(); l1++) {
+            lfoWidget(l1)->restoreParams(ix);
+        }
+        for (l1 = 0; l1 < seqWidgetCount(); l1++) {
+            seqWidget(l1)->restoreParams(ix);
+        }
+    }
+    else {
+        if (restoreModType == 'A') {
+            arpWidget(restoreModIx)->restoreParams(ix);
+        }
+        else if (restoreModType == 'L') {
+            lfoWidget(restoreModIx)->restoreParams(ix);
+        }
+        else if (restoreModType == 'S') {
+            seqWidget(restoreModIx)->restoreParams(ix);
+        }
+    }
+
+    globStoreWidget->setDispState(ix, 1, restoreModWindowIndex);
+    restoreRequest = -1;
+}
+
+void Engine::removeParStores(int ix)
+{
+    int l1;
+    for (l1 = 0; l1 < arpWidgetCount(); l1++) {
+        arpWidget(l1)->parStore->list.removeAt(ix);
+    }
+    for (l1 = 0; l1 < lfoWidgetCount(); l1++) {
+        lfoWidget(l1)->parStore->list.removeAt(ix);
+    }
+    for (l1 = 0; l1 < seqWidgetCount(); l1++) {
+        seqWidget(l1)->parStore->list.removeAt(ix);
+    }
+}
+
+void Engine::updateGlobRestoreTimeModule(int windowIndex)
+{
+    restoreModType = globStoreWidget->timeModuleBox
+                        ->itemText(windowIndex).at(0);
+
+    restoreModIx = moduleWindowList.at(windowIndex)->widget()
+                        ->property("widgetID").toInt();
+}
