@@ -4,7 +4,7 @@
  *
  * @section LICENSE
  *
- *      Copyright 2009, 2010, 2011 <qmidiarp-devel@lists.sourceforge.net>
+ *      Copyright 2009, 2010, 2011, 2012 <qmidiarp-devel@lists.sourceforge.net>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -37,6 +37,7 @@
 #include "midiseq.h"
 #include "seqwidget.h"
 #include "groovewidget.h"
+#include "globstore.h"
 
 /*!
  * @brief Core Engine Thread. Instantiates SeqDriver and JackDriver.
@@ -68,12 +69,20 @@ class Engine : public QThread  {
     int midiLearnID, midiLearnWindowID, midiLearnModuleID;
     bool midiLearnFlag;
     bool useMidiClock;
+    int restoreModIx;
+    QChar restoreModType;
+    int restoreModWindowIndex;
+    int restoreRequest;
+    int restoreModule;
+    int restoreTick;
 
     //From SeqDriver
     int schedDelayTicks;
     int nextMinLfoTick;
     int nextMinSeqTick;
     int nextMinArpTick;
+    int currentTick;
+    int requestTick;
     QVector<Sample> lfoData;
     Sample seqSample;
     bool sendLogEvents;
@@ -87,12 +96,13 @@ class Engine : public QThread  {
     bool midiControllable;
     bool status;
     bool ready;
+    GlobStore *globStoreWidget;
     GrooveWidget *grooveWidget;
     JackDriver *jackSync;
     DriverBase *driver;
 
   public:
-    Engine(GrooveWidget *p_grooveWidget, int p_portCount, bool p_alsamidi, QWidget* parent=0);
+    Engine(GlobStore *p_globStore, GrooveWidget *p_grooveWidget, int p_portCount, bool p_alsamidi, QWidget* parent=0);
     ~Engine();
     int getPortCount();
     bool isModified();
@@ -132,6 +142,7 @@ class Engine : public QThread  {
     SeqWidget *seqWidget(int index);
     int getClientId();
     void setTempo(int bpm);
+    void sendGroove();
 
   signals:
 /**
@@ -141,11 +152,14 @@ class Engine : public QThread  {
  * @param tick Set to the tick value at which the event was received
  */
     void midiEventReceived(MidiEvent ev, int tick);
+    void restoreSig(int ix);
+    void indicPercentSig(int p);
+    void updateCursorSig(QChar modtype, int ix, int pos);
 
   public slots:
     void setStatus(bool);
 /**
- * @brief This function is used to set the modified flag, which is queried before
+ * @brief Sets the modified flag, which is queried before
  * loading a new session file or quitting qmidiarp.
  *
  * @param m Set to True if parameter modifications are present
@@ -154,18 +168,73 @@ class Engine : public QThread  {
     void updatePatternPresets(const QString& n, const QString& p, int index);
     void sendController(int ccnumber, int channel, int value);
     void learnController(int ccnumber, int channel);
+/**
+ * @brief Slot for MidiControl::setMidiLearn(). Sets Engine into MIDI Learn status for
+ * moduleID and controlID.
+ * 
+ * Engine will then wait for an incoming controller event and trigger the
+ * attribution by calling MidiControl::appendMidiCC().
+ * 
+ * @param moduleWindowID dockWidget ID of the module
+ * @param moduleID ID of the module (index in the moduleWidgetList)
+ * @param controlID ID of the controllable widget requesting MIDI learn
+ */
     void setMidiLearn(int moduleWindowID, int moduleID, int controlID);
     void setMidiControllable(bool on);
     void setCompactStyle(bool on);
     void setGrooveTick(int grooveTick);
     void setGrooveVelocity(int grooveVelocity);
     void setGrooveLength(int grooveLength);
-    void sendGroove();
     void setSendLogEvents(bool on);
     void setUseMidiClock(bool on);
     bool eventCallback(MidiEvent inEv);
     void echoCallback(bool echo_from_trig);
     void resetTicks(int curtick);
+    void indicPercent(int p);
+    void updateCursor(QChar modtype, int ix, int pos);
+/*!
+* @brief causes the module of type Engine::restoreModType at position
+* Engine::restoreModIx in the moduleWidgetList to restore its current
+* parameters from its ParStore::list at index ix. If Engine::restoreModType
+* is set to -1, all modules are called for restore.
+*
+* @param ix ParStore::list index from which all module parameters are to be restored
+*/
+    void restore(int ix);
+/*!
+* @brief causes all modules to remove their entries in the ParStore::list
+* at index ix
+*
+* @param ix ParStore::list index at which module parameters are to be removed
+*/
+    void removeParStores(int ix);
+/*!
+* @brief causes all modules to store their current parameters in their
+* ParStore::list at index ix
+*
+* @param ix ParStore::list index at which all module parameters are to be stored
+*/
+    void globStore(int ix);
+/*!
+* @brief causes Engine to call Engine::restore() when the timing and
+* restore type conditions are met
+* @param windowIndex Engine::moduleWindowList index of the module which should
+* restore its parameters. If windowIndex is set to -1, all modules are called
+* @param ix ParStore::list index from which all module parameters are to be restored
+*/
+    void requestRestore(int windowIndex, int ix);
+/*!
+* @brief signal slot for GlobStore::updateGlobRestoreTimeModule signal
+*
+* Makes the module with index windowIndex trigger global store switches when its cursor
+* reaches the end. It determines the type (Arp, LFO, Seq) and the index of the selected
+* module the module storage lists and stores these in local variables.
+*
+* @param windowIndex Engine::moduleWindowList index of the module to become switch
+* trigger when its cursor reaches the end of the pattern
+*/
+    void updateGlobRestoreTimeModule(int windowIndex);
+
 };
 
 #endif
