@@ -4,7 +4,7 @@
  *
  * @section LICENSE
  *
- *      Copyright 2009, 2010, 2011 <qmidiarp-devel@lists.sourceforge.net>
+ *      Copyright 2009, 2010, 2011, 2012 <qmidiarp-devel@lists.sourceforge.net>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -30,18 +30,16 @@
 #include <QPushButton>
 #include <QComboBox>
 #include <QSpinBox>
-#include <QStringList>
 #include <QGroupBox>
 
 #include "groovewidget.h"
 #include "slider.h"
+#include "main.h"
 
 
 GrooveWidget::GrooveWidget(QWidget *parent) : QWidget(parent)
 {
-    QStringList midiCCNames;
-    midiCCNames << "Tick" << "Velocity" << "Length" << "unknown";
-    midiControl = new MidiControl(midiCCNames);
+    midiControl = new MidiControl(this);
     midiControl->ID = -1;
     midiControl->parentDockID = -1;
 
@@ -51,19 +49,19 @@ GrooveWidget::GrooveWidget(QWidget *parent) : QWidget(parent)
             tr("&Shift"), this);
     connect(grooveTick, SIGNAL(valueChanged(int)),
             this, SLOT(updateGrooveTick(int)));
-    midiControl->addMidiLearnMenu(grooveTick, 0);
+    midiControl->addMidiLearnMenu("Tick", grooveTick, 0);
 
     grooveVelocity = new Slider(-100, 100, 1, 10, 0, Qt::Horizontal,
             tr("&Velocity"), this);
     connect(grooveVelocity, SIGNAL(valueChanged(int)),
             this, SLOT(updateGrooveVelocity(int)));
-    midiControl->addMidiLearnMenu(grooveVelocity, 1);
+    midiControl->addMidiLearnMenu("Velocity", grooveVelocity, 1);
 
     grooveLength = new Slider(-100, 100, 1, 10, 0, Qt::Horizontal,
             tr("&Length"), this);
     connect(grooveLength, SIGNAL(valueChanged(int)),
             this, SLOT(updateGrooveLength(int)));
-    midiControl->addMidiLearnMenu(grooveLength, 2);
+    midiControl->addMidiLearnMenu("Length", grooveLength, 2);
 
     GrooveWidgetLayout->addWidget(grooveTick);
     GrooveWidgetLayout->addWidget(grooveVelocity);
@@ -120,3 +118,34 @@ void GrooveWidget::handleController(int ccnumber, int channel, int value)
         }
     }
 }
+void GrooveWidget::readData(QXmlStreamReader& xml)
+{
+    while (!xml.atEnd()) {
+        xml.readNext();
+        if (xml.isEndElement())
+            break;
+        if (xml.name() == "tick")
+            grooveTick->setValue(xml.readElementText().toInt());
+        else if (xml.name() == "velocity")
+            grooveVelocity->setValue(xml.readElementText().toInt());
+        else if (xml.name() == "length")
+            grooveLength->setValue(xml.readElementText().toInt());
+        else if (xml.isStartElement() && (xml.name() == "midiControllers")) {
+            midiControl->readData(xml);
+        }
+        else skipXmlElement(xml);
+    }
+}
+
+void GrooveWidget::writeData(QXmlStreamWriter& xml)
+{
+    xml.writeStartElement("groove");
+        xml.writeTextElement("tick",
+            QString::number(grooveTick->value()));
+        xml.writeTextElement("velocity",
+            QString::number(grooveVelocity->value()));
+        xml.writeTextElement("length",
+            QString::number(grooveLength->value()));
+        midiControl->writeData(xml);
+    xml.writeEndElement();
+}
