@@ -4,7 +4,7 @@
  *
  * @section LICENSE
  *
- *      Copyright 2009, 2010, 2011 <qmidiarp-devel@lists.sourceforge.net>
+ *      Copyright 2009, 2010, 2011, 2012 <qmidiarp-devel@lists.sourceforge.net>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -32,6 +32,8 @@
 #include <QComboBox>
 #include <QCheckBox>
 #include <QSpinBox>
+#include <QXmlStreamReader>
+#include <QXmlStreamWriter>
 #include "slider.h"
 #include "midicontrol.h"
 
@@ -57,6 +59,21 @@ class GrooveWidget : public QWidget
     ~GrooveWidget();
     MidiControl *midiControl;
 
+/*!
+* @brief This function reads all parameters of this module from an XML stream
+* passed by the caller, i.e. MainWindow.
+*
+* @param xml QXmlStreamWriter to read from
+*/
+    void readData(QXmlStreamReader& xml);
+/*!
+* @brief This function writes all parameters of this module to an XML stream
+* passed by the caller, i.e. MainWindow.
+*
+* @param xml QXmlStreamWriter to write to
+*/
+    void writeData(QXmlStreamWriter& xml);
+
   signals:
     void newGrooveVelocity(int);
     void newGrooveTick(int);
