@@ -4,7 +4,7 @@
  *
  * @section LICENSE
  *
- *      Copyright 2009, 2010, 2011 <qmidiarp-devel@lists.sourceforge.net>
+ *      Copyright 2009, 2010, 2011, 2012 <qmidiarp-devel@lists.sourceforge.net>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -27,7 +27,6 @@
 #include <QGroupBox>
 #include <QLabel>
 #include <QMessageBox>
-#include <QStringList>
 
 #include "midilfo.h"
 #include "lfowidget.h"
@@ -49,10 +48,10 @@ LfoWidget::LfoWidget(MidiLfo *p_midiWorker, int portCount, bool compactStyle,
     QWidget(parent), midiWorker(p_midiWorker), modified(false)
 {
     int l1;
-    QStringList midiCCNames;
-    midiCCNames << "MuteToggle" << "Amplitude" << "Offset" << "WaveForm" << "Frequency"
-                << "RecordToggle"<< "Resolution"<< "Size" << "LoopMode" << "unknown";
-    midiControl = new MidiControl(midiCCNames);
+
+    parStore = new ParStore();
+
+    midiControl = new MidiControl(this);
 
     manageBox = new ManageBox("LFO:", true, this);
 
@@ -77,9 +76,16 @@ LfoWidget::LfoWidget(MidiLfo *p_midiWorker, int portCount, bool compactStyle,
     enableTrigByKbdLabel->setBuddy(enableTrigByKbd);
     enableTrigByKbd->setToolTip(tr("Retrigger sequence when a new note is received"));
 
+    QLabel *enableTrigLegatoLabel = new QLabel(tr("&Legato"),inBox);
+    enableTrigLegato = new QCheckBox(this);
+    connect(enableTrigLegato, SIGNAL(toggled(bool)), this, SLOT(updateTrigLegato(bool)));
+    enableTrigLegatoLabel->setBuddy(enableTrigLegato);
+    enableTrigLegato->setToolTip(tr("Retrigger / restart upon new legato note as well"));
+
     enableNoteOff->setChecked(false);
     enableRestartByKbd->setChecked(false);
     enableTrigByKbd->setChecked(false);
+    enableTrigLegato->setChecked(false);
 
     QLabel *ccnumberInLabel = new QLabel(tr("MIDI &CC#"), inBox);
     ccnumberInBox = new QSpinBox(inBox);
@@ -108,8 +114,10 @@ LfoWidget::LfoWidget(MidiLfo *p_midiWorker, int portCount, bool compactStyle,
     inBoxLayout->addWidget(enableRestartByKbd, 2, 1);
     inBoxLayout->addWidget(enableTrigByKbdLabel, 3, 0);
     inBoxLayout->addWidget(enableTrigByKbd, 3, 1);
-    inBoxLayout->addWidget(chInLabel, 4, 0);
-    inBoxLayout->addWidget(chIn, 4, 1);
+    inBoxLayout->addWidget(enableTrigLegatoLabel, 4, 0);
+    inBoxLayout->addWidget(enableTrigLegato, 4, 1);
+    inBoxLayout->addWidget(chInLabel, 5, 0);
+    inBoxLayout->addWidget(chIn, 5, 1);
     if (compactStyle) {
         inBoxLayout->setSpacing(1);
         inBoxLayout->setMargin(2);
@@ -124,7 +132,7 @@ LfoWidget::LfoWidget(MidiLfo *p_midiWorker, int portCount, bool compactStyle,
     muteOut = new QCheckBox(this);
     connect(muteOut, SIGNAL(toggled(bool)), this, SLOT(setMuted(bool)));
     muteLabel->setBuddy(muteOut);
-    midiControl->addMidiLearnMenu(muteOut, 0);
+    midiControl->addMidiLearnMenu("MuteToggle", muteOut, 0);
 
 
     QLabel *ccnumberLabel = new QLabel(tr("MIDI &CC#"), portBox);
@@ -204,7 +212,7 @@ LfoWidget::LfoWidget(MidiLfo *p_midiWorker, int portCount, bool compactStyle,
 
     connect(waveFormBox, SIGNAL(activated(int)), this,
             SLOT(updateWaveForm(int)));
-    midiControl->addMidiLearnMenu(waveFormBox, 3);
+    midiControl->addMidiLearnMenu("WaveForm", waveFormBox, 3);
 
 
     QLabel *freqBoxLabel = new QLabel(tr("&Frequency"),
@@ -222,7 +230,7 @@ LfoWidget::LfoWidget(MidiLfo *p_midiWorker, int portCount, bool compactStyle,
     freqBox->setMinimumContentsLength(3);
     connect(freqBox, SIGNAL(activated(int)), this,
             SLOT(updateFreq(int)));
-    midiControl->addMidiLearnMenu(freqBox, 4);
+    midiControl->addMidiLearnMenu("Frequency", freqBox, 4);
 
     QLabel *resBoxLabel = new QLabel(tr("&Resolution"),
             waveBox);
@@ -237,7 +245,7 @@ LfoWidget::LfoWidget(MidiLfo *p_midiWorker, int portCount, bool compactStyle,
     resBox->setMinimumContentsLength(3);
     connect(resBox, SIGNAL(activated(int)), this,
             SLOT(updateRes(int)));
-    midiControl->addMidiLearnMenu(resBox, 6);
+    midiControl->addMidiLearnMenu("Resolution", resBox, 6);
 
     QLabel *sizeBoxLabel = new QLabel(tr("&Length"), waveBox);
     sizeBox = new QComboBox(waveBox);
@@ -251,7 +259,7 @@ LfoWidget::LfoWidget(MidiLfo *p_midiWorker, int portCount, bool compactStyle,
     sizeBox->setMinimumContentsLength(3);
     connect(sizeBox, SIGNAL(activated(int)), this,
             SLOT(updateSize(int)));
-    midiControl->addMidiLearnMenu(sizeBox, 7);
+    midiControl->addMidiLearnMenu("Size", sizeBox, 7);
 
     loopBox = new QComboBox(waveBox);
     names.clear();
@@ -262,7 +270,7 @@ LfoWidget::LfoWidget(MidiLfo *p_midiWorker, int portCount, bool compactStyle,
     loopBox->setMinimumContentsLength(5);
     connect(loopBox, SIGNAL(activated(int)), this,
             SLOT(updateLoop(int)));
-    midiControl->addMidiLearnMenu(loopBox, 8);
+    midiControl->addMidiLearnMenu("LoopMode", loopBox, 8);
 
 
     QLabel *recordButtonLabel = new QLabel(tr("Re&cord"), waveBox);
@@ -273,20 +281,20 @@ LfoWidget::LfoWidget(MidiLfo *p_midiWorker, int portCount, bool compactStyle,
     recordButton->setDefaultAction(recordAction);
     recordButtonLabel->setBuddy(recordButton);
     connect(recordAction, SIGNAL(toggled(bool)), this, SLOT(setRecord(bool)));
-    midiControl->addMidiLearnMenu(recordButton, 5);
+    midiControl->addMidiLearnMenu("RecordToggle", recordButton, 5);
 
     amplitude = new Slider(0, 127, 1, 8, 64, Qt::Horizontal,
             tr("&Amplitude"), waveBox);
     connect(amplitude, SIGNAL(valueChanged(int)), this,
             SLOT(updateAmp(int)));
-    midiControl->addMidiLearnMenu(amplitude, 1);
+    midiControl->addMidiLearnMenu("Amplitude", amplitude, 1);
 
 
     offset = new Slider(0, 127, 1, 8, 0, Qt::Horizontal,
             tr("&Offset"), waveBox);
     connect(offset, SIGNAL(valueChanged(int)), this,
             SLOT(updateOffs(int)));
-    midiControl->addMidiLearnMenu(offset, 2);
+    midiControl->addMidiLearnMenu("Offset", offset, 2);
 
     QVBoxLayout* sliderLayout = new QVBoxLayout;
     sliderLayout->addWidget(amplitude);
@@ -361,6 +369,8 @@ void LfoWidget::writeData(QXmlStreamWriter& xml)
                 midiWorker->restartByKbd));
             xml.writeTextElement("trigByKbd", QString::number(
                 midiWorker->trigByKbd));
+            xml.writeTextElement("trigLegato", QString::number(
+                midiWorker->trigLegato));
             xml.writeTextElement("channel", QString::number(
                 midiWorker->chIn));
             xml.writeTextElement("ccnumber", QString::number(
@@ -417,6 +427,8 @@ void LfoWidget::writeData(QXmlStreamWriter& xml)
 
         midiControl->writeData(xml);
 
+        parStore->writeData(xml);
+
     xml.writeEndElement();
 }
 
@@ -442,6 +454,8 @@ void LfoWidget::readData(QXmlStreamReader& xml)
                     enableRestartByKbd->setChecked(xml.readElementText().toInt());
                 else if (xml.name() == "trigByKbd")
                     enableTrigByKbd->setChecked(xml.readElementText().toInt());
+                else if (xml.name() == "trigLegato")
+                    enableTrigLegato->setChecked(xml.readElementText().toInt());
                 if (xml.name() == "channel") {
                     tmp = xml.readElementText().toInt();
                     chIn->setCurrentIndex(tmp);
@@ -551,6 +565,9 @@ void LfoWidget::readData(QXmlStreamReader& xml)
         else if (xml.isStartElement() && (xml.name() == "midiControllers")) {
             midiControl->readData(xml);
         }
+        else if (xml.isStartElement() && (xml.name() == "globalStores")) {
+            parStore->readData(xml);
+        }
         else skipXmlElement(xml);
     }
     waveFormBox->setCurrentIndex(wvtmp);
@@ -558,23 +575,6 @@ void LfoWidget::readData(QXmlStreamReader& xml)
     modified = false;
 }
 
-void LfoWidget::skipXmlElement(QXmlStreamReader& xml)
-{
-    if (xml.isStartElement()) {
-        qWarning("Unknown Element in XML File: %s",qPrintable(xml.name().toString()));
-        while (!xml.atEnd()) {
-            xml.readNext();
-
-            if (xml.isEndElement())
-                break;
-
-            if (xml.isStartElement()) {
-                skipXmlElement(xml);
-            }
-        }
-    }
-}
-
 void LfoWidget::loadWaveForms()
 {
     waveForms << tr("Sine") << tr("Saw up") << tr("Triangle")
@@ -622,6 +622,12 @@ void LfoWidget::updateEnableTrigByKbd(bool on)
     modified = true;
 }
 
+void LfoWidget::updateTrigLegato(bool on)
+{
+    midiWorker->trigLegato = on;
+    modified = true;
+}
+
 void LfoWidget::updateWaveForm(int val)
 {
     if (val > 5) return;
@@ -813,6 +819,61 @@ void LfoWidget::setModified(bool m)
     midiControl->setModified(m);
 }
 
+void LfoWidget::storeParams(int ix, bool empty)
+{
+    parStore->temp.empty = empty;
+    parStore->temp.muteOut = muteOut->isChecked();
+    parStore->temp.chIn = chIn->currentIndex();
+    parStore->temp.ccnumberIn = ccnumberInBox->value();
+    parStore->temp.ccnumber = ccnumberBox->value();
+    parStore->temp.channelOut = channelOut->currentIndex();
+    parStore->temp.portOut = portOut->currentIndex();
+    parStore->temp.res = resBox->currentIndex();
+    parStore->temp.size = sizeBox->currentIndex();
+    parStore->temp.loopMode = loopBox->currentIndex();
+    parStore->temp.freq = freqBox->currentIndex();
+    parStore->temp.ampl = amplitude->value();
+    parStore->temp.offs = offset->value();
+    parStore->temp.waveForm = waveFormBox->currentIndex();
+
+    parStore->temp.wave = getCustomWave();
+    parStore->temp.muteMask = midiWorker->muteMask;
+
+    parStore->tempToList(ix);
+}
+
+void LfoWidget::restoreParams(int ix)
+{
+    if (parStore->list.at(ix).empty) return;
+    midiWorker->customWave = parStore->list.at(ix).wave;
+    midiWorker->muteMask = parStore->list.at(ix).muteMask;
+    sizeBox->setCurrentIndex(parStore->list.at(ix).size);
+    midiWorker->updateSize(sizeBox->currentText().toInt());
+
+    midiWorker->updateResolution(lfoResValues[parStore->list.at(ix).res]);
+    midiWorker->updateWaveForm(parStore->list.at(ix).waveForm);
+    midiWorker->updateFrequency(lfoFreqValues[parStore->list.at(ix).freq]);
+    freqBox->setCurrentIndex(parStore->list.at(ix).freq);
+    resBox->setCurrentIndex(parStore->list.at(ix).res);
+    waveFormBox->setCurrentIndex(parStore->list.at(ix).waveForm);
+    loopBox->setCurrentIndex(parStore->list.at(ix).loopMode);
+
+    muteOut->setChecked(parStore->list.at(ix).muteOut);
+    chIn->setCurrentIndex(parStore->list.at(ix).chIn);
+    updateChIn(parStore->list.at(ix).chIn);
+    ccnumberInBox->setValue(parStore->list.at(ix).ccnumberIn);
+    ccnumberBox->setValue(parStore->list.at(ix).ccnumber);
+    channelOut->setCurrentIndex(parStore->list.at(ix).channelOut);
+    updateChannelOut(parStore->list.at(ix).channelOut);
+    setPortOut(parStore->list.at(ix).portOut);
+    updatePortOut(parStore->list.at(ix).portOut);
+    updateLoop(parStore->list.at(ix).loopMode);
+    updateWaveForm(parStore->list.at(ix).waveForm);
+    offset->setValue(parStore->list.at(ix).offs);
+    amplitude->setValue(parStore->list.at(ix).ampl);
+    midiWorker->setFramePtr(0);
+}
+
 void LfoWidget::copyParamsFrom(LfoWidget *fromWidget)
 {
     int tmp;
@@ -820,6 +881,7 @@ void LfoWidget::copyParamsFrom(LfoWidget *fromWidget)
     enableNoteOff->setChecked(fromWidget->enableNoteOff->isChecked());
     enableRestartByKbd->setChecked(fromWidget->enableRestartByKbd->isChecked());
     enableTrigByKbd->setChecked(fromWidget->enableTrigByKbd->isChecked());
+    enableTrigLegato->setChecked(fromWidget->enableTrigLegato->isChecked());
 
     tmp = fromWidget->chIn->currentIndex();
     chIn->setCurrentIndex(tmp);
