@@ -4,7 +4,7 @@
  *
  * @section LICENSE
  *
- *      Copyright 2009, 2010, 2011 <qmidiarp-devel@lists.sourceforge.net>
+ *      Copyright 2009, 2010, 2011, 2012 <qmidiarp-devel@lists.sourceforge.net>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -40,10 +40,8 @@
 #include "lfoscreen.h"
 #include "midicontrol.h"
 #include "managebox.h"
+#include "parstore.h"
 
-/*! This array holds the currently available resolution values.
- */
-const int lfoResValues[9] = {1, 2, 4, 8, 16, 32, 64, 96, 192};
 /*! This array holds the currently available frequency values.
  */
 const int lfoFreqValues[14] = {1, 2, 4, 8, 16, 24, 32, 64, 96, 128, 160, 192, 224, 256};
@@ -70,16 +68,6 @@ class LfoWidget : public QWidget
     bool modified;              /**< Is set to True if unsaved parameter modifications exist */
     bool lastMute;              /**< Contains the mute state of the last waveForm point modified by mouse click*/
 /*!
-* @brief This function allows ignoring one XML element in the XML stream
-* passed by the caller.
-*
-* It also advances the stream read-in. It is used to
-* ignore unknown elements for both-ways-compatibility
-*
-* @param xml reference to QXmlStreamReader containing the open XML stream
-*/
-    void skipXmlElement(QXmlStreamReader& xml);
-/*!
 * @brief This function populates the LfoWidget::waveForms list with
 * waveform names.
 *
@@ -120,6 +108,7 @@ class LfoWidget : public QWidget
             bool mutedAdd = false, QWidget* parent=0);
     ~LfoWidget();
 
+    ParStore *parStore;
     MidiControl *midiControl;
     LfoScreen *screen;
     ManageBox *manageBox;
@@ -135,6 +124,7 @@ class LfoWidget : public QWidget
     QCheckBox *enableNoteOff;
     QCheckBox *enableRestartByKbd;
     QCheckBox *enableTrigByKbd;
+    QCheckBox *enableTrigLegato;
     Slider *frequency, *amplitude, *offset;
     QAction *recordAction;
     QComboBox *waveFormBox, *freqBox;
@@ -164,6 +154,20 @@ class LfoWidget : public QWidget
 */
     void writeData(QXmlStreamWriter& xml);
 /*!
+* @brief This function stores some module parameters in a parameter
+* list object
+*
+* @param Position index in the parameter list
+*/
+    void storeParams(int ix, bool empty = false);
+/*!
+* @brief This function restores some module parameters from the parameter
+* list object
+*
+* @param Position index in the parameter list
+*/
+    void restoreParams(int ix);
+/*!
 * @brief This function copies all LFO module GUI parameters from
 * fromWidget
 *
@@ -211,6 +215,7 @@ class LfoWidget : public QWidget
     void updateEnableNoteOff(bool on);
     void updateEnableRestartByKbd(bool on);
     void updateEnableTrigByKbd(bool on);
+    void updateTrigLegato(bool on);
 
 /*!
 * @brief Slot for the LfoWidget::waveFormBox combobox setting the waveform
