@@ -3,7 +3,7 @@
  *
  *   @section LICENSE
  *
- *      Copyright 2009, 2010, 2011 <qmidiarp-devel@lists.sourceforge.net>
+ *      Copyright 2009, 2010, 2011, 2012 <qmidiarp-devel@lists.sourceforge.net>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -24,10 +24,21 @@
 #ifndef __MAIN_H
 #define __MAIN_H
 
+#include <QXmlStreamReader>
+
 #include "midievent.h"
 
 extern QString global_jack_session_uuid;
-
+/*!
+* @brief allows ignoring one XML element in the XML stream
+* passed by the caller.
+*
+* It also advances the stream read-in. It is used to
+* ignore unknown elements for both-ways-compatibility
+*
+* @param xml reference to QXmlStreamReader containing the open XML stream
+*/
+extern void skipXmlElement(QXmlStreamReader& xml);
 
 #define CT_FOOTSW       0x40
 
