@@ -4,7 +4,7 @@
  *
  * @section LICENSE
  *
- *      Copyright 2009, 2010, 2011 <qmidiarp-devel@lists.sourceforge.net>
+ *      Copyright 2009, 2010, 2011, 2012 <qmidiarp-devel@lists.sourceforge.net>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -22,7 +22,6 @@
  *      MA 02110-1301, USA.
  *
  */
-#include <QBoxLayout>
 #include <QDir>
 #include <QFile>
 #include <QFileDialog>
@@ -35,6 +34,7 @@
 #include <QSocketNotifier>
 #include <QStringList>
 #include <QSpinBox>
+#include <QStyle>
 #include <QTableWidget>
 #include <QTextStream>
 #include <QXmlStreamReader>
@@ -54,6 +54,7 @@
 #include "pixmaps/seqadd.xpm"
 #include "pixmaps/settings.xpm"
 #include "pixmaps/eventlog.xpm"
+#include "pixmaps/globtog.xpm"
 #include "pixmaps/groovetog.xpm"
 #include "pixmaps/play.xpm"
 #include "pixmaps/midiclock.xpm"
@@ -83,11 +84,28 @@ MainWindow::MainWindow(int p_portCount, bool p_alsamidi)
     grooveWindow->setWidget(grooveWidget);
     grooveWindow->setObjectName("grooveWidget");
     grooveWindow->setVisible(true);
-    addDockWidget(Qt::BottomDockWidgetArea, grooveWindow);
 
-    engine = new Engine(grooveWidget, p_portCount, alsaMidi, this);
+    globStore = new GlobStore(this);
+    globStoreWindow = new QDockWidget(tr("Global Store"), this);
+    globStoreWindow->setFeatures(QDockWidget::DockWidgetClosable
+            | QDockWidget::DockWidgetMovable
+            | QDockWidget::DockWidgetFloatable);
+    globStoreWindow->setWidget(globStore);
+    globStoreWindow->setObjectName("globStore");
+    globStoreWindow->setVisible(true);
+
+    engine = new Engine(globStore, grooveWidget, p_portCount, alsaMidi, this);
     if (!alsaMidi) connect(engine->driver, SIGNAL(jsEvent(int)), this, SLOT(jsAction(int)));
 
+    connect(globStore, SIGNAL(globStore(int)), engine,
+            SLOT(globStore(int)));
+    connect(globStore, SIGNAL(requestRestore(int, int)), engine,
+            SLOT(requestRestore(int, int)));
+    connect(globStore, SIGNAL(updateGlobRestoreTimeModule(int)), engine,
+            SLOT(updateGlobRestoreTimeModule(int)));
+    connect(globStore, SIGNAL(removeParStores(int)), engine,
+            SLOT(removeParStores(int)));
+
     midiCCTable = new MidiCCTable(engine, this);
 
     logWidget = new LogWidget(this);
@@ -97,7 +115,6 @@ MainWindow::MainWindow(int p_portCount, bool p_alsamidi)
             | QDockWidget::DockWidgetFloatable);
     logWindow->setWidget(logWidget);
     logWindow->setObjectName("logWidget");
-    addDockWidget(Qt::BottomDockWidgetArea, logWindow);
     qRegisterMetaType<MidiEvent>("MidiEvent");
     connect(engine, SIGNAL(midiEventReceived(MidiEvent, int)),
             logWidget, SLOT(appendEvent(MidiEvent, int)));
@@ -105,6 +122,10 @@ MainWindow::MainWindow(int p_portCount, bool p_alsamidi)
     connect(logWidget, SIGNAL(sendLogEvents(bool)),
             engine, SLOT(setSendLogEvents(bool)));
 
+    addDockWidget(Qt::BottomDockWidgetArea, globStoreWindow);
+    addDockWidget(Qt::BottomDockWidgetArea, grooveWindow);
+    addDockWidget(Qt::BottomDockWidgetArea, logWindow);
+
     passWidget = new PassWidget(engine, p_portCount, this);
 
     addArpAction = new QAction(QIcon(arpadd_xpm), tr("&New Arp..."), this);
@@ -206,6 +227,12 @@ MainWindow::MainWindow(int p_portCount, bool p_alsamidi)
                     "View|Settings")));
     connect(viewSettingsAction, SIGNAL(triggered()), passWidget, SLOT(show()));
 
+    QAction* viewGlobAction = globStoreWindow->toggleViewAction();
+    viewGlobAction->setIcon(QIcon(globtog_xpm));
+    viewGlobAction->setText(tr("&Global Store"));
+    viewGlobAction->setShortcut(QKeySequence(tr("Ctrl+$",
+                    "View|GlobalStore")));
+
     QMenuBar *menuBar = new QMenuBar;
     QMenu *fileMenu = new QMenu(tr("&File"), this);
     QMenu *viewMenu = new QMenu(tr("&View"), this);
@@ -229,6 +256,7 @@ MainWindow::MainWindow(int p_portCount, bool p_alsamidi)
     viewMenu->addAction(viewLogAction);
     viewMenu->addAction(viewGrooveAction);
     viewMenu->addAction(viewSettingsAction);
+    viewMenu->addAction(viewGlobAction);
     viewMenu->addAction(tr("&MIDI Controllers..."),
             this, SLOT(showMidiCCDialog()));
 
@@ -255,6 +283,7 @@ MainWindow::MainWindow(int p_portCount, bool p_alsamidi)
     controlToolBar = new QToolBar(tr("&Control Toolbar"), this);
     controlToolBar->addAction(viewLogAction);
     controlToolBar->addAction(viewGrooveAction);
+    controlToolBar->addAction(viewGlobAction);
     controlToolBar->addSeparator();
     controlToolBar->addAction(addArpAction);
     controlToolBar->addAction(addLfoAction);
@@ -279,8 +308,8 @@ MainWindow::MainWindow(int p_portCount, bool p_alsamidi)
     addToolBar(controlToolBar);
 
     setWindowIcon(QPixmap(qmidiarp2_xpm));
-    QWidget *centWidget = new QWidget(this);
-    setCentralWidget(centWidget);
+
+    setCentralWidget(new QWidget(this));
     setDockNestingEnabled(true);
     updateWindowTitle();
 
@@ -359,7 +388,7 @@ void MainWindow::seqNew()
     }
 }
 
-void MainWindow::addArp(const QString& name)
+void MainWindow::addArp(const QString& name, bool fromfile)
 {
     int count, widgetID;
     MidiArp *midiWorker = new MidiArp();
@@ -382,6 +411,13 @@ void MainWindow::addArp(const QString& name)
     moduleWidget->manageBox->ID = widgetID;
     moduleWidget->midiControl->ID = widgetID;
 
+    // if the module is added at a time when global stores are already
+    // present we fill up the new global parameter storage list with dummies
+    // and tag them empty
+    if (!fromfile) for (int l1 = 0; l1 < (globStore->widgetList.count() - 1); l1++) {
+        moduleWidget->storeParams(l1, true);
+    }
+
     engine->addMidiArp(midiWorker);
     engine->addArpWidget(moduleWidget);
     engine->sendGroove();
@@ -389,14 +425,17 @@ void MainWindow::addArp(const QString& name)
     count = engine->moduleWindowCount();
     moduleWidget->manageBox->parentDockID = count;
     moduleWidget->midiControl->parentDockID = count;
+    moduleWidget->setProperty("widgetID", widgetID);
     appendDock(moduleWidget, name, count);
 
     checkIfFirstModule();
 }
 
-void MainWindow::addLfo(const QString& name)
+void MainWindow::addLfo(const QString& p_name, bool fromfile, int clonefrom)
 {
     int widgetID, count;
+    QString name;
+
     MidiLfo *midiWorker = new MidiLfo();
     LfoWidget *moduleWidget = new LfoWidget(midiWorker,
             engine->getPortCount(), passWidget->compactStyle,
@@ -410,23 +449,45 @@ void MainWindow::addLfo(const QString& name)
             engine, SLOT(setMidiLearn(int, int, int)));
 
     widgetID = engine->lfoWidgetCount();
+    if (clonefrom >= 0) {
+        name = engine->lfoWidget(clonefrom)->manageBox->name + "_0";
+        moduleWidget->copyParamsFrom(engine->lfoWidget(clonefrom));
+    }
+    else name = p_name;
+
     moduleWidget->manageBox->name = name;
     moduleWidget->manageBox->ID = widgetID;
     moduleWidget->midiControl->ID = widgetID;
 
+    // if the module is added at a time when global stores are already
+    // present we fill up the new global parameter storage list with dummies
+    // and tag them empty
+    if (!fromfile) for (int l1 = 0; l1 < (globStore->widgetList.count() - 1); l1++) {
+        moduleWidget->storeParams(l1, true);
+    }
+
+    if (clonefrom >= 0) {
+        midiWorker->reverse = engine->lfoWidget(clonefrom)->getReverse();
+        midiWorker->setFramePtr(engine->lfoWidget(clonefrom)->getFramePtr());
+        midiWorker->nextTick = engine->lfoWidget(clonefrom)->getNextTick();
+    }
+
     engine->addMidiLfo(midiWorker);
     engine->addLfoWidget(moduleWidget);
     count = engine->moduleWindowCount();
     moduleWidget->manageBox->parentDockID = count;
     moduleWidget->midiControl->parentDockID = count;
+    moduleWidget->setProperty("widgetID", widgetID);
     appendDock(moduleWidget, name, count);
 
     checkIfFirstModule();
 }
 
-void MainWindow::addSeq(const QString& name)
+void MainWindow::addSeq(const QString& p_name, bool fromfile, int clonefrom)
 {
     int widgetID, count;
+    QString name;
+    
     MidiSeq *midiWorker = new MidiSeq();
     SeqWidget *moduleWidget = new SeqWidget(midiWorker,
             engine->getPortCount(), passWidget->compactStyle,
@@ -439,15 +500,35 @@ void MainWindow::addSeq(const QString& name)
             engine, SLOT(setMidiLearn(int, int, int)));
 
     widgetID = engine->seqWidgetCount();
+    if (clonefrom >= 0) {
+        name = engine->seqWidget(clonefrom)->manageBox->name + "_0";
+        moduleWidget->copyParamsFrom(engine->seqWidget(clonefrom));
+    }
+    else name = p_name;
+    
     moduleWidget->manageBox->name = name;
     moduleWidget->manageBox->ID = widgetID;
     moduleWidget->midiControl->ID = widgetID;
 
+    // if the module is added at a time when global stores are already
+    // present we fill up the new global parameter storage list with dummies
+    // and tag them empty
+    if (!fromfile) for (int l1 = 0; l1 < (globStore->widgetList.count() - 1); l1++) {
+        moduleWidget->storeParams(l1, true);
+    }
+    
+    if (clonefrom >= 0) {
+        midiWorker->reverse = engine->seqWidget(clonefrom)->getReverse();
+        midiWorker->setCurrentIndex(engine->seqWidget(clonefrom)->getCurrentIndex());
+        midiWorker->nextTick = engine->seqWidget(clonefrom)->getNextTick();
+    }
+
     engine->addMidiSeq(midiWorker);
     engine->addSeqWidget(moduleWidget);
     count = engine->moduleWindowCount();
     moduleWidget->manageBox->parentDockID = count;
     moduleWidget->midiControl->parentDockID = count;
+    moduleWidget->setProperty("widgetID", widgetID);
     appendDock(moduleWidget, name, count);
 
     checkIfFirstModule();
@@ -455,69 +536,12 @@ void MainWindow::addSeq(const QString& name)
 
 void MainWindow::cloneLfo(int ID)
 {
-    int widgetID, count;
-    MidiLfo *midiWorker = new MidiLfo();
-    LfoWidget *moduleWidget = new LfoWidget(midiWorker,
-            engine->getPortCount(), passWidget->compactStyle,
-            passWidget->mutedAdd, this);
-    connect(moduleWidget->manageBox, SIGNAL(moduleRemove(int)),
-            this, SLOT(removeLfo(int)));
-    connect(moduleWidget->manageBox, SIGNAL(moduleClone(int)), this, SLOT(cloneLfo(int)));
-    connect(moduleWidget->manageBox, SIGNAL(dockRename(const QString&, int)),
-            this, SLOT(renameDock(const QString&, int)));
-    connect(moduleWidget->midiControl, SIGNAL(setMidiLearn(int, int, int)),
-            engine, SLOT(setMidiLearn(int, int, int)));
-
-    widgetID = engine->lfoWidgetCount();
-    moduleWidget->manageBox->name = engine->lfoWidget(ID)->manageBox->name + "_0";
-    moduleWidget->manageBox->ID = widgetID;
-    moduleWidget->midiControl->ID = widgetID;
-
-    moduleWidget->copyParamsFrom(engine->lfoWidget(ID));
-    midiWorker->reverse = engine->lfoWidget(ID)->getReverse();
-    midiWorker->setFramePtr(engine->lfoWidget(ID)->getFramePtr());
-    midiWorker->nextTick = engine->lfoWidget(ID)->getNextTick();
-    engine->addMidiLfo(midiWorker);
-    engine->addLfoWidget(moduleWidget);
-    count = engine->moduleWindowCount();
-    moduleWidget->manageBox->parentDockID = count;
-    moduleWidget->midiControl->parentDockID = count;
-    appendDock(moduleWidget, moduleWidget->manageBox->name, count);
-
+    addLfo("", false, ID);
 }
 
 void MainWindow::cloneSeq(int ID)
 {
-    int widgetID, count;
-    QString name;
-    MidiSeq *midiWorker = new MidiSeq();
-
-    SeqWidget *moduleWidget = new SeqWidget(midiWorker,
-            engine->getPortCount(), passWidget->compactStyle,
-            passWidget->mutedAdd, this);
-    connect(moduleWidget->manageBox, SIGNAL(moduleRemove(int)), this, SLOT(removeSeq(int)));
-    connect(moduleWidget->manageBox, SIGNAL(moduleClone(int)), this, SLOT(cloneSeq(int)));
-    connect(moduleWidget->manageBox, SIGNAL(dockRename(const QString&, int)),
-            this, SLOT(renameDock(const QString&, int)));
-    connect(moduleWidget->midiControl, SIGNAL(setMidiLearn(int, int, int)),
-            engine, SLOT(setMidiLearn(int, int, int)));
-
-    widgetID = engine->seqWidgetCount();
-    moduleWidget->manageBox->name = engine->seqWidget(ID)->manageBox->name + "_0";
-    moduleWidget->manageBox->ID = widgetID;
-    moduleWidget->midiControl->ID = widgetID;
-
-    moduleWidget->copyParamsFrom(engine->seqWidget(ID));
-    midiWorker->reverse = engine->lfoWidget(ID)->getReverse();
-    midiWorker->setCurrentIndex(engine->seqWidget(ID)->getCurrentIndex());
-    midiWorker->nextTick = engine->seqWidget(ID)->getNextTick();
-    engine->addMidiSeq(midiWorker);
-    engine->addSeqWidget(moduleWidget);
-    count = engine->moduleWindowCount();
-    moduleWidget->manageBox->parentDockID = count;
-    moduleWidget->midiControl->parentDockID = count;
-    appendDock(moduleWidget, moduleWidget->manageBox->name, count);
-
+    addSeq("", false, ID);
 }
 
 void MainWindow::appendDock(QWidget *moduleWidget, const QString &name, int count)
@@ -532,6 +556,8 @@ void MainWindow::appendDock(QWidget *moduleWidget, const QString &name, int coun
 
     if (count) tabifyDockWidget(engine->moduleWindow(count - 1), moduleWindow);
     engine->addModuleWindow(moduleWindow);
+    globStore->addModule(name);
+
 }
 
 void MainWindow::renameDock(const QString& name, int parentDockID)
@@ -547,6 +573,7 @@ void MainWindow::removeArp(int index)
 
     parentDockID = arpWidget->manageBox->parentDockID;
     QDockWidget *dockWidget = engine->moduleWindow(parentDockID);
+    globStore->removeModule(parentDockID);
 
     engine->removeMidiArp(arpWidget->getMidiWorker());
     engine->removeArpWidget(arpWidget);
@@ -563,6 +590,7 @@ void MainWindow::removeLfo(int index)
 
     parentDockID = lfoWidget->manageBox->parentDockID;
     QDockWidget *dockWidget = engine->moduleWindow(parentDockID);
+    globStore->removeModule(parentDockID);
 
     engine->removeMidiLfo(lfoWidget->getMidiWorker());
     engine->removeLfoWidget(lfoWidget);
@@ -579,6 +607,7 @@ void MainWindow::removeSeq(int index)
 
     parentDockID = seqWidget->manageBox->parentDockID;
     QDockWidget *dockWidget = engine->moduleWindow(parentDockID);
+    globStore->removeModule(parentDockID);
 
     engine->removeMidiSeq(seqWidget->getMidiWorker());
     engine->removeSeqWidget(seqWidget);
@@ -593,6 +622,11 @@ void MainWindow::clear()
     updateTransportStatus(false);
     jackSyncToggle(false);
 
+    for (int l1 = globStore->widgetList.count() - 1; l1 > 0; l1--) {
+        globStore->removeLocation(l1);
+    }
+    globStore->setDispState(0, 0);
+    globStore->midiControl->ccList.clear();
     while (engine->midiArpCount()) {
         removeArp(engine->midiArpCount() - 1);
     }
@@ -606,6 +640,7 @@ void MainWindow::clear()
     }
 
     grooveWidget->midiControl->ccList.clear();
+
 }
 
 void MainWindow::fileNew()
@@ -677,6 +712,8 @@ void MainWindow::openFile(const QString& fn)
                     readFilePartModules(xml);
                 else if (xml.isStartElement() && (xml.name() == "GUI"))
                     readFilePartGUI(xml);
+                else if (xml.isStartElement() && (xml.name() == "globalstorage"))
+                    globStore->readData(xml);
                 else skipXmlElement(xml);
             }
         }
@@ -719,47 +756,53 @@ void MainWindow::readFilePartGlobal(QXmlStreamReader& xml)
                 else skipXmlElement(xml);
             }
         }
-        else if (xml.isStartElement() && (xml.name() == "groove")) {
-            while (!xml.atEnd()) {
-                xml.readNext();
-                if (xml.isEndElement())
-                    break;
-                if (xml.name() == "tick")
-                    grooveWidget->grooveTick->setValue(xml.readElementText().toInt());
-                else if (xml.name() == "velocity")
-                    grooveWidget->grooveVelocity->setValue(xml.readElementText().toInt());
-                else if (xml.name() == "length")
-                    grooveWidget->grooveLength->setValue(xml.readElementText().toInt());
-                else if (xml.isStartElement() && (xml.name() == "midiControllers")) {
-                    grooveWidget->midiControl->readData(xml);
-                }
-                else skipXmlElement(xml);
-            }
-        }
+        else if (xml.isStartElement() && (xml.name() == "groove"))
+            grooveWidget->readData(xml);
         else skipXmlElement(xml);
     }
 }
 
 void MainWindow::readFilePartModules(QXmlStreamReader& xml)
 {
+    int count = 0;
+
+
     while (!xml.atEnd()) {
         xml.readNext();
         if (xml.isEndElement())
             break;
         if (xml.isStartElement() && (xml.name() == "Arp")) {
-            addArp("Arp:" + xml.attributes().value("name").toString());
+            addArp("Arp:" + xml.attributes().value("name").toString(), true);
             engine->arpWidget(engine->midiArpCount() - 1)
                     ->readData(xml);
+            count++;
+            if (count == 1) {
+                for (int l1 = 0; l1 < engine->arpWidget(0)->parStore->list.count(); l1++) {
+                    globStore->addLocation();
+                }
+            }
         }
         else if (xml.isStartElement() && (xml.name() == "LFO")) {
-            addLfo("LFO:" + xml.attributes().value("name").toString());
+            addLfo("LFO:" + xml.attributes().value("name").toString(), true);
             engine->lfoWidget(engine->midiLfoCount() - 1)
                     ->readData(xml);
+            count++;
+            if (count == 1) {
+                for (int l1 = 0; l1 < engine->lfoWidget(0)->parStore->list.count(); l1++) {
+                    globStore->addLocation();
+                }
+            }
         }
         else if (xml.isStartElement() && (xml.name() == "Seq")) {
-            addSeq("Seq:" + xml.attributes().value("name").toString());
+            addSeq("Seq:" + xml.attributes().value("name").toString(), true);
             engine->seqWidget(engine->midiSeqCount() - 1)
                     ->readData(xml);
+            count++;
+            if (count == 1) {
+                for (int l1 = 0; l1 < engine->seqWidget(0)->parStore->list.count(); l1++) {
+                    globStore->addLocation();
+                }
+            }
         }
         else skipXmlElement(xml);
     }
@@ -779,23 +822,6 @@ void MainWindow::readFilePartGUI(QXmlStreamReader& xml)
     }
 }
 
-void MainWindow::skipXmlElement(QXmlStreamReader& xml)
-{
-    if (xml.isStartElement()) {
-        qWarning("Unknown Element in XML File: %s",qPrintable(xml.name().toString()));
-        while (!xml.atEnd()) {
-            xml.readNext();
-
-            if (xml.isEndElement())
-                break;
-
-            if (xml.isStartElement()) {
-                skipXmlElement(xml);
-            }
-        }
-    }
-}
-
 void MainWindow::fileSave()
 {
     if (filename.isEmpty())
@@ -836,24 +862,16 @@ bool MainWindow::saveFile()
             xml.writeTextElement("midiControlEnabled",
                 QString::number((int)passWidget->cbuttonCheck->isChecked()));
             xml.writeTextElement("midiClockEnabled",
-                QString::number((int)engine->driver->useMidiClock));
+                QString::number((int)midiClockAction->isChecked()));
             xml.writeTextElement("jackSyncEnabled",
-                QString::number((int)engine->driver->useJackSync));
+                QString::number((int)jackSyncAction->isChecked()));
             xml.writeTextElement("forwardUnmatched",
-                QString::number((int)engine->driver->forwardUnmatched));
+                QString::number((int)passWidget->forwardCheck->isChecked()));
             xml.writeTextElement("forwardPort",
-                QString::number(engine->driver->portUnmatched));
+                QString::number(passWidget->portUnmatchedSpin->currentIndex()));
         xml.writeEndElement();
 
-        xml.writeStartElement("groove");
-            xml.writeTextElement("tick",
-                QString::number(engine->grooveTick));
-            xml.writeTextElement("velocity",
-                QString::number(engine->grooveVelocity));
-            xml.writeTextElement("length",
-                QString::number(engine->grooveLength));
-            grooveWidget->midiControl->writeData(xml);
-        xml.writeEndElement();
+        grooveWidget->writeData(xml);
 
     xml.writeEndElement();
 
@@ -883,6 +901,8 @@ bool MainWindow::saveFile()
         xml.writeTextElement("windowState", saveState().toHex());
     xml.writeEndElement();
 
+    globStore->writeData(xml);
+
     xml.writeEndElement();
     xml.writeEndDocument();
 
@@ -1009,13 +1029,6 @@ void MainWindow::setGUIforExtSync(bool on)
 {
     runAction->setDisabled(on);
     tempoSpin->setDisabled(on);
-    /*
-    addArpAction->setDisabled(on);
-    addLfoAction->setDisabled(on);
-    addSeqAction->setDisabled(on);
-    fileOpenAction->setDisabled(on);
-    fileRecentlyOpenedFiles->setDisabled(on);
-    * */
 }
 
 bool MainWindow::checkRcFile()
