@@ -4,7 +4,7 @@
  *
  * @section LICENSE
  *
- *      Copyright 2009, 2010, 2011 <qmidiarp-devel@lists.sourceforge.net>
+ *      Copyright 2009, 2010, 2011, 2012 <qmidiarp-devel@lists.sourceforge.net>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -28,7 +28,9 @@
 
 #include <QAction>
 #include <QApplication>
+#include <QBoxLayout>
 #include <QCloseEvent>
+#include <QComboBox>
 #include <QDockWidget>
 #include <QMessageBox>
 #include <QMainWindow>
@@ -39,6 +41,7 @@
 #include "engine.h"
 #include "midicctable.h"
 #include "passwidget.h"
+#include "globstore.h"
 #include "groovewidget.h"
 #include "config.h"
 
@@ -49,14 +52,18 @@
 
 static const char ABOUTMSG[] =
             "<html> <p><b><big>" APP_NAME " " PACKAGE_VERSION "</big></b></p>"
-            "<p>(C) 2009-2011 Frank Kober<br/>"
+            "<p>(C) 2009-2012 Frank Kober<br/>"
             "(C) 2011 Nedko Arnaudov<br/>"
             "(C) 2009 Guido Scholz<br/>"
             "(C) 2002-2003 Matthias Nagorni (SuSE AG Nuremberg)<br/></p>"
+            "<p><b>Translations</b><br/>"
+            "Pavel Fric<br/>"
+            "Pedro Lopez-Cabanillas<br/>"
+            "Robert Dietrich<br/></p>"
             "<p>For getting support please type <b>man qmidiarp</b> or go to<br/>"
             "<a href=\"http://qmidiarp.sourceforge.net\">"
             "http://qmidiarp.sourceforge.net</a></p>"
-            APP_NAME " is licensed under the GPL.</b></p></html>";
+            APP_NAME " is licensed under the GPLv2.</b></p></html>";
 
 /*!
  * The MainWindow class is the main UI that holds functions to manage global
@@ -79,12 +86,13 @@ class MainWindow : public QMainWindow
     PassWidget *passWidget;
     GrooveWidget *grooveWidget;
     LogWidget *logWidget;
+    GlobStore *globStore;
     Engine *engine;
     MidiCCTable *midiCCTable;
     QString lastDir, filename;
     QStringList patternNames, patternPresets;
     QStringList recentFiles;
-    QDockWidget *logWindow, *grooveWindow, *passWindow;
+    QDockWidget *logWindow, *grooveWindow, *passWindow, *globStoreWindow;
 
     QToolBar *controlToolBar, *fileToolBar;
     QAction *runAction, *addArpAction;
@@ -131,7 +139,6 @@ class MainWindow : public QMainWindow
 *
 */
     bool isModified();
-
 /*!
 * @brief This function creates and adds a new MidiArp to Engine.
 *
@@ -141,8 +148,9 @@ class MainWindow : public QMainWindow
 * Engine::MidiArpList.
 *
 * @param name Name attribute of the created arpeggiator module
+* @param fromfile Set to True if module is added by a file read
 */
-    void addArp(const QString&);
+    void addArp(const QString&, bool fromfile = false);
 /*!
 * @brief This function creates and adds a new MidiLfo to Engine.
 *
@@ -152,8 +160,11 @@ class MainWindow : public QMainWindow
 * Engine::MidiLfoList.
 *
 * @param name Name attribute of this LFO module
+* @param fromfile Set to True if module is added by a file read
+* @param clonefrom Set to the ID to clone this module from,
+* -1 for a new module (default)
 */
-    void addLfo(const QString&);
+    void addLfo(const QString&, bool fromfile = false, int clonefrom = -1);
 /*!
 * @brief This function creates and adds a new MidiSeq to Engine.
 *
@@ -163,8 +174,11 @@ class MainWindow : public QMainWindow
 * Engine::MidiSeqList.
 *
 * @param name Name attribute of this arpeggiator module
+* @param fromfile Set to True if module is added by a file read
+* @param clonefrom Set to the ID to clone this module from,
+* -1 for a new module (default)
 */
-    void addSeq(const QString&);
+    void addSeq(const QString&, bool fromfile = false, int clonefrom = -1);
 /*!
 * @brief This function wraps the given widget in a QDockWidget and adds
 * it to the list in Engine.
@@ -302,17 +316,6 @@ class MainWindow : public QMainWindow
  * @return True if installation succeeded.
 */
     bool installSignalHandlers();
-/*!
-* @brief This function allows ignoring one XML element in the XML stream
-* passed by the caller.
-*
-* It advances the stream read-in. It is used to
-* ignore unknown elements for both-ways-compatibility
-*
-* @param xml Reference to QXmlStreamReader containing the open XML stream
-*/
-    void skipXmlElement(QXmlStreamReader& xml);
-
 
   protected:
 /*!
@@ -428,12 +431,18 @@ class MainWindow : public QMainWindow
 */
     void removeSeq(int index);
 /*!
-* @brief This function duplicates and adds a MidiSeq to the Engine.
+* @brief This function duplicates and adds a MidiLfo to the Engine.
 *
 * @param ID List ID of the module to copy
 */
     void cloneLfo(int ID);
+/*!
+* @brief This function duplicates and adds a MidiSeq to the Engine.
+*
+* @param ID List ID of the module to copy
+*/
     void cloneSeq(int ID);
+
     void helpAbout();
     void helpAboutQt();
 /*! @brief Slot for tempo spinBox changes.
