@@ -4,7 +4,7 @@
  *
  * @section LICENSE
  *
- *      Copyright 2009, 2010, 2011 <qmidiarp-devel@lists.sourceforge.net>
+ *      Copyright 2009, 2010, 2011, 2012 <qmidiarp-devel@lists.sourceforge.net>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -75,7 +75,6 @@ class MidiArp : public QWidget  {
     int newGrooveTick, grooveTick, grooveVelocity, grooveLength;
     int randomTick, randomVelocity, randomLength;
     double stepWidth, len, vel;
-    double minStepWidth; /*!< Holds the minimum step width of the pattern for quantization purposes*/
     QVector<int> sustainBuffer; /*!< Holds released note values when MidiArp::sustain is True */
     QVector<int> latchBuffer;   /*!< Holds released note values when MidiArp::latch_mode is True */
     QTimer *latchTimer;         /*!< Is started when a note is released and causes MidiArp::purgeLatchBuffer after
@@ -110,7 +109,6 @@ class MidiArp : public QWidget  {
     int noteBufPtr;     /*!< Pointer to the currently active note buffer copy */
     int noteCount;      /*!< The number of notes in the MidiArp::notes buffer */
     int patternLen;     /*!< Length of the arp text pattern */
-    int patternMaxIndex;/*!< Maximum number of stacked notes in the pattern */
     int noteOfs;        /*!< The current index in a chord. @see repeatPatternThroughChord */
     int releaseNoteCount; /*!< The number of notes currently in release stage */
 
@@ -182,17 +180,6 @@ class MidiArp : public QWidget  {
  * @return Number of notes present at the MIDI input port.
  */
     void removeNote(int *noteptr, int tick, int keep_rel);
-/**
- * @brief This function calculates the index of the next arpeggio
- * step and revolves it if necessary.
- *
- * The next step depends on the MidiArp::repeatPatternThrough chord mode.
- * The pattern index can be simply reset to zero if the reset flag is
- * set to True.
- * @param reset Set to True in order to set the pattern index to zero
- * @return True if the pattern index is now zero
- */
-    bool advancePatternIndex(bool reset);
 /**
  * @brief This function removes a note inside the MidiArp::notes input
  * note buffer.
@@ -233,21 +220,29 @@ class MidiArp : public QWidget  {
     bool isMuted;   /*!< Mute state set by ArpWidget */
     bool restartByKbd; /*!< If True, restart pattern at 0 upon new received note, set by ArpWidget */
     bool trigByKbd; /*!< If True, trigger current note tick by tick of received note, set by ArpWidget */
-    bool gotKbdTrig;
+    bool trigLegato; /*!< If True, trigger and restart upon legato input notes as well */
+    bool gotKbdTrig; /*!< Is set when a keyboard trigger is activated by a note */
     int repeatPatternThroughChord; /*!< Repeat mode "Static", "Up", "Down", set by ArpWidget */
     double attack_time;/*!< Attack time in seconds, set by ArpWidget */
     double release_time;/*!< Release time in seconds, set by ArpWidget */
     int randomTickAmp; /*!< Amplitude of timing randomization, set by ArpWidget */
     int randomVelocityAmp; /*!< Amplitude of velocity randomization, set by ArpWidget */
     int randomLengthAmp; /*!< Amplitude of length randomization, set by ArpWidget */
+
     QString pattern; /*!< Holds the the arpeggio pattern text */
+    int maxOctave;      /*!< Maximum octave shift found in the pattern */
+    int minOctave;      /*!< Minimum octave shift found in the pattern */
+    double minStepWidth; /*!< Minimum step width of the pattern for quantization purposes*/
+    double nSteps;      /*!< Musical length of the pattern in beats */
+    int nPoints;        /*!< Number of steps to be played out */
+    int patternMaxIndex;/*!< Maximum number of stacked notes in the pattern */
 
     QVector<int> returnNote; /*!< Holds the notes of the currently active arpeggio step */
     QVector<int> returnVelocity; /*!< Holds the velocities of the currently active arpeggio step */
     int returnTick; /*!< Holds the time in internal ticks of the currently active arpeggio step */
     int returnLength; /*!< Holds the note length of the currently active arpeggio step */
     int returnIsNew;
-    int nextTick;
+    int nextTick; /*!< Holds the next tick at which note events will be played out */
 
   public:
     MidiArp();
@@ -265,6 +260,17 @@ class MidiArp : public QWidget  {
  * @param on Set to True to suppress data output to the Driver
  */
     void setMuted(bool);
+/**
+ * @brief This function calculates the index of the next arpeggio
+ * step and revolves it if necessary.
+ *
+ * The next step depends on the MidiArp::repeatPatternThrough chord mode.
+ * The pattern index can be simply reset to zero if the reset flag is
+ * set to True.
+ * @param reset Set to True in order to set the pattern index to zero
+ * @return True if the pattern index is now zero
+ */
+    bool advancePatternIndex(bool reset);
 /**
  * @brief This function does the actions related to a newly received event.
  *
