@@ -4,7 +4,7 @@
  *
  * @section LICENSE
  *
- *      Copyright 2009, 2010, 2011 <qmidiarp-devel@lists.sourceforge.net>
+ *      Copyright 2009, 2010, 2011, 2012 <qmidiarp-devel@lists.sourceforge.net>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -100,6 +100,7 @@ class MidiSeq : public QObject  {
     bool enableVelIn;
     bool restartByKbd;
     bool trigByKbd;
+    bool trigLegato; /*!< If True, trigger and restart upon legato input notes as well */
     bool enableLoop;
     bool gotKbdTrig;
     bool restartFlag; /*!< Signals frameptr reset on next getNextFrame() call */
@@ -111,6 +112,7 @@ class MidiSeq : public QObject  {
     int size, res, waveFormIndex;
     int currentRecStep;
     int loopMarker;
+    int nPoints;        /*!< Number of steps to be played out */
     int nextTick;
     int nOctaves;
     int baseOctave;
@@ -160,19 +162,31 @@ class MidiSeq : public QObject  {
  * @see MidiSeq::toggleMutePoint(), MidiSeq::setMutePoint()
  */
     void setCustomWavePoint(double mouseX, double mouseY);
+/*! @brief This function sets the MidiSeq::loopMarker member variable
+ * used as a supplemental return point within the sequence.
+ *
+ * @param ix Absolute pattern index position of the marker. NOTE:
+ * If ix is negative, the
+ * marker acts to the left, whereas it acts to the right when mouseX is
+ * positive. If ix is zero, the loopMarker will be removed.
+ * @see MidiSeq::toggleMutePoint(), MidiSeq::setMutePoint(),
+ * MidiSeq::setLoopMarkerMouse()
+ */
+    void setLoopMarker(int ix);
 /*! @brief This function sets the MidiSeq::loopMarker member variable
  * used as a supplemental return point within the sequence. It is called
  *  by SeqWidget::mousePressed().
  * The normalized mouse coordinates are scaled to the waveform size and
- * resolution.
+ * resolution and then MidiSeq::setLoopMarker() is called
  *
  * @param mouseX Normalized horizontal location of the mouse on the
  * SeqScreen (0.0 ... 1.0). If the mouseX parameter is negative, the
  * marker acts to the left, whereas it acts to the right when mouseX is
  * positive. If mouseX is zero, the loopMarker will be removed.
- * @see MidiSeq::toggleMutePoint(), MidiSeq::setMutePoint()
+ * @see MidiSeq::toggleMutePoint(), MidiSeq::setMutePoint(),
+ * MidiSeq::setLoopMarker()
  */
-    void setLoopMarker(double mouseX);
+    void setLoopMarkerMouse(double mouseX);
 /*! @brief This function sets the mute state of one point of the
  * MidiSeq::muteMask array to the given state.
  *
