@@ -4,7 +4,7 @@
  *
  * @section LICENSE
  *
- *      Copyright 2009, 2010, 2011 <qmidiarp-devel@lists.sourceforge.net>
+ *      Copyright 2009, 2010, 2011, 2012 <qmidiarp-devel@lists.sourceforge.net>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -25,7 +25,6 @@
 #include <QLabel>
 #include <QBoxLayout>
 #include <QGridLayout>
-#include <QStringList>
 #include <QGroupBox>
 #include <QInputDialog>
 #include <QMessageBox>
@@ -40,10 +39,10 @@ SeqWidget::SeqWidget(MidiSeq *p_midiWorker, int portCount, bool compactStyle,
     QWidget(parent), midiWorker(p_midiWorker), modified(false)
 {
     int l1;
-    QStringList midiCCNames;
-    midiCCNames << "MuteToggle" << "Velocity" << "NoteLength"
-                << "RecordToggle" << "Resolution"<< "Size" << "LoopMode" << "unknown";
-    midiControl = new MidiControl(midiCCNames);
+
+    parStore = new ParStore();
+
+    midiControl = new MidiControl(this);
 
     manageBox = new ManageBox("Seq:", true, this);
 
@@ -116,11 +115,18 @@ SeqWidget::SeqWidget(MidiSeq *p_midiWorker, int portCount, bool compactStyle,
     enableTrigByKbdLabel->setBuddy(enableTrigByKbd);
     enableTrigByKbd->setToolTip(tr("Retrigger sequence when a new note is received"));
 
+    QLabel *enableTrigLegatoLabel = new QLabel(tr("&Legato"),inBox);
+    enableTrigLegato = new QCheckBox(this);
+    connect(enableTrigLegato, SIGNAL(toggled(bool)), this, SLOT(updateTrigLegato(bool)));
+    enableTrigLegatoLabel->setBuddy(enableTrigLegato);
+    enableTrigLegato->setToolTip(tr("Retrigger / restart upon new legato note as well"));
+
     enableNoteIn->setChecked(true);
     enableNoteOff->setChecked(false);
     enableVelIn->setChecked(true);
     enableRestartByKbd->setChecked(false);
     enableTrigByKbd->setChecked(false);
+    enableTrigLegato->setChecked(false);
 
     QLabel *chInLabel = new QLabel(tr("&Channel"), inBox);
     chIn = new QComboBox(inBox);
@@ -140,8 +146,10 @@ SeqWidget::SeqWidget(MidiSeq *p_midiWorker, int portCount, bool compactStyle,
     inBoxLayout->addWidget(enableRestartByKbd, 3, 1);
     inBoxLayout->addWidget(enableTrigByKbdLabel, 4, 0);
     inBoxLayout->addWidget(enableTrigByKbd, 4, 1);
-    inBoxLayout->addWidget(chInLabel, 5, 0);
-    inBoxLayout->addWidget(chIn, 5, 1);
+    inBoxLayout->addWidget(enableTrigLegatoLabel, 5, 0);
+    inBoxLayout->addWidget(enableTrigLegato, 5, 1);
+    inBoxLayout->addWidget(chInLabel, 6, 0);
+    inBoxLayout->addWidget(chIn, 6, 1);
     if (compactStyle) {
         inBoxLayout->setSpacing(1);
         inBoxLayout->setMargin(2);
@@ -157,7 +165,7 @@ SeqWidget::SeqWidget(MidiSeq *p_midiWorker, int portCount, bool compactStyle,
     muteOut = new QCheckBox(this);
     connect(muteOut, SIGNAL(toggled(bool)), this, SLOT(setMuted(bool)));
     muteLabel->setBuddy(muteOut);
-    midiControl->addMidiLearnMenu(muteOut, 0);
+    midiControl->addMidiLearnMenu("MuteToggle", muteOut, 0);
 
     QLabel *portLabel = new QLabel(tr("&Port"), portBox);
     portOut = new QComboBox(portBox);
@@ -230,7 +238,7 @@ SeqWidget::SeqWidget(MidiSeq *p_midiWorker, int portCount, bool compactStyle,
     loopBox->setMinimumContentsLength(5);
     connect(loopBox, SIGNAL(activated(int)), this,
             SLOT(updateLoop(int)));
-    midiControl->addMidiLearnMenu(loopBox, 6);
+    midiControl->addMidiLearnMenu("LoopMode", loopBox, 6);
 
     QLabel *recordButtonLabel = new QLabel(tr("Re&cord"), seqBox);
     recordAction = new QAction(QIcon(seqrecord_xpm), tr("Re&cord"), seqBox);
@@ -240,7 +248,7 @@ SeqWidget::SeqWidget(MidiSeq *p_midiWorker, int portCount, bool compactStyle,
     recordButton->setDefaultAction(recordAction);
     recordButtonLabel->setBuddy(recordButton);
     connect(recordAction, SIGNAL(toggled(bool)), this, SLOT(setRecord(bool)));
-    midiControl->addMidiLearnMenu(recordButton, 3);
+    midiControl->addMidiLearnMenu("RecordToggle", recordButton, 3);
 
     QLabel *resBoxLabel = new QLabel(tr("&Resolution"),
             seqBox);
@@ -255,7 +263,7 @@ SeqWidget::SeqWidget(MidiSeq *p_midiWorker, int portCount, bool compactStyle,
     resBox->setMinimumContentsLength(3);
     connect(resBox, SIGNAL(activated(int)), this,
             SLOT(updateRes(int)));
-    midiControl->addMidiLearnMenu(resBox, 4);
+    midiControl->addMidiLearnMenu("Resolution", resBox, 4);
 
     QLabel *sizeBoxLabel = new QLabel(tr("&Length"), seqBox);
     sizeBox = new QComboBox(seqBox);
@@ -268,7 +276,7 @@ SeqWidget::SeqWidget(MidiSeq *p_midiWorker, int portCount, bool compactStyle,
     sizeBox->setMinimumContentsLength(3);
     connect(sizeBox, SIGNAL(activated(int)), this,
             SLOT(updateSize(int)));
-    midiControl->addMidiLearnMenu(sizeBox, 5);
+    midiControl->addMidiLearnMenu("Size", sizeBox, 5);
 
     //temporarily hide these elements until multiple patterns are implemented
     waveFormBox->setEnabled(false);
@@ -280,14 +288,14 @@ SeqWidget::SeqWidget(MidiSeq *p_midiWorker, int portCount, bool compactStyle,
             tr("Veloc&ity"), seqBox);
     connect(velocity, SIGNAL(valueChanged(int)), this,
             SLOT(updateVelocity(int)));
-    midiControl->addMidiLearnMenu(velocity, 1);
+    midiControl->addMidiLearnMenu("Velocity", velocity, 1);
 
 
     notelength = new Slider(0, 127, 1, 16, 64, Qt::Horizontal,
             tr("N&ote Length"), seqBox);
     connect(notelength, SIGNAL(valueChanged(int)), this,
             SLOT(updateNoteLength(int)));
-    midiControl->addMidiLearnMenu(notelength, 2);
+    midiControl->addMidiLearnMenu("NoteLength", notelength, 2);
 
     transpose = new Slider(-24, 24, 1, 2, 0, Qt::Horizontal,
             tr("&Transpose"), seqBox);
@@ -373,6 +381,8 @@ void SeqWidget::writeData(QXmlStreamWriter& xml)
                 midiWorker->restartByKbd));
             xml.writeTextElement("trigByKbd", QString::number(
                 midiWorker->trigByKbd));
+            xml.writeTextElement("trigLegato", QString::number(
+                midiWorker->trigLegato));
             xml.writeTextElement("channel", QString::number(
                 midiWorker->chIn));
         xml.writeEndElement();
@@ -425,6 +435,8 @@ void SeqWidget::writeData(QXmlStreamWriter& xml)
 
         midiControl->writeData(xml);
 
+        parStore->writeData(xml);
+
     xml.writeEndElement();
 }
 
@@ -464,6 +476,8 @@ void SeqWidget::readData(QXmlStreamReader& xml)
                     enableRestartByKbd->setChecked(xml.readElementText().toInt());
                 else if (xml.name() == "trigByKbd")
                     enableTrigByKbd->setChecked(xml.readElementText().toInt());
+                else if (xml.name() == "trigLegato")
+                    enableTrigLegato->setChecked(xml.readElementText().toInt());
                 else if (xml.name() == "channel") {
                     tmp = xml.readElementText().toInt();
                     chIn->setCurrentIndex(tmp);
@@ -567,7 +581,7 @@ void SeqWidget::readData(QXmlStreamReader& xml)
                 }
                 else if (xml.name() == "loopmarker") {
                     tmp = xml.readElementText().toInt();
-                    midiWorker->loopMarker = tmp;
+                    midiWorker->setLoopMarker(tmp);
                     screen->setLoopMarker(tmp);
                 }
                 else skipXmlElement(xml);
@@ -576,6 +590,9 @@ void SeqWidget::readData(QXmlStreamReader& xml)
         else if (xml.isStartElement() && (xml.name() == "midiControllers")) {
             midiControl->readData(xml);
         }
+        else if (xml.isStartElement() && (xml.name() == "globalStores")) {
+            parStore->readData(xml);
+        }
         else skipXmlElement(xml);
     }
     waveFormBox->setCurrentIndex(wvtmp);
@@ -583,23 +600,6 @@ void SeqWidget::readData(QXmlStreamReader& xml)
     modified = false;
 }
 
-void SeqWidget::skipXmlElement(QXmlStreamReader& xml)
-{
-    if (xml.isStartElement()) {
-        qWarning("Unknown Element in XML File: %s",qPrintable(xml.name().toString()));
-        while (!xml.atEnd()) {
-            xml.readNext();
-
-            if (xml.isEndElement())
-                break;
-
-            if (xml.isStartElement()) {
-                skipXmlElement(xml);
-            }
-        }
-    }
-}
-
 void SeqWidget::loadWaveForms()
 {
     waveForms << tr("Custom");
@@ -659,6 +659,12 @@ void SeqWidget::updateEnableTrigByKbd(bool on)
     modified = true;
 }
 
+void SeqWidget::updateTrigLegato(bool on)
+{
+    midiWorker->trigLegato = on;
+    modified = true;
+}
+
 void SeqWidget::updateNoteLength(int val)
 {
     midiWorker->notelength = val + val;
@@ -765,7 +771,7 @@ void SeqWidget::mousePressed(double mouseX, double mouseY, int buttons)
     if (mouseY < 0) {
         if (mouseX < 0) mouseX = 0;
         if (buttons == 2) mouseX = - mouseX;
-        midiWorker->setLoopMarker(mouseX);
+        midiWorker->setLoopMarkerMouse(mouseX);
         screen->setLoopMarker(midiWorker->loopMarker);
         modified = true;
         return;
@@ -865,6 +871,61 @@ void SeqWidget::updateDispVert(int mode)
     modified = true;
 }
 
+void SeqWidget::storeParams(int ix, bool empty)
+{
+    parStore->temp.empty = empty;
+    parStore->temp.muteOut = muteOut->isChecked();
+    parStore->temp.chIn = chIn->currentIndex();
+    parStore->temp.channelOut = channelOut->currentIndex();
+    parStore->temp.portOut = portOut->currentIndex();
+    parStore->temp.res = resBox->currentIndex();
+    parStore->temp.size = sizeBox->currentIndex();
+    parStore->temp.loopMode = loopBox->currentIndex();
+    parStore->temp.notelen = notelength->value();
+    parStore->temp.transp = transpose->value();
+    parStore->temp.vel = velocity->value();
+    parStore->temp.dispVertical = dispVertical;
+    parStore->temp.loopMode = loopBox->currentIndex();
+    parStore->temp.wave = getCustomWave();
+    parStore->temp.muteMask = midiWorker->muteMask;
+    parStore->temp.loopMarker = getLoopMarker();
+
+    parStore->tempToList(ix);
+}
+
+void SeqWidget::restoreParams(int ix)
+{
+    if (parStore->list.at(ix).empty) return;
+    midiWorker->customWave = parStore->list.at(ix).wave;
+    midiWorker->muteMask = parStore->list.at(ix).muteMask;
+    sizeBox->setCurrentIndex(parStore->list.at(ix).size);
+    midiWorker->size = sizeBox->currentText().toInt();
+    midiWorker->res = seqResValues[parStore->list.at(ix).res];
+    midiWorker->resizeAll();
+    midiWorker->setLoopMarker(parStore->list.at(ix).loopMarker);
+    screen->setLoopMarker(parStore->list.at(ix).loopMarker);
+
+    resBox->setCurrentIndex(parStore->list.at(ix).res);
+    waveFormBox->setCurrentIndex(parStore->list.at(ix).waveForm);
+    loopBox->setCurrentIndex(parStore->list.at(ix).loopMode);
+    notelength->setValue(parStore->list.at(ix).notelen);
+    transpose->setValue(parStore->list.at(ix).transp);
+    velocity->setValue(parStore->list.at(ix).vel);
+    setDispVert(parStore->list.at(ix).dispVertical);
+
+    muteOut->setChecked(parStore->list.at(ix).muteOut);
+    chIn->setCurrentIndex(parStore->list.at(ix).chIn);
+    updateChIn(parStore->list.at(ix).chIn);
+    channelOut->setCurrentIndex(parStore->list.at(ix).channelOut);
+    updateChannelOut(parStore->list.at(ix).channelOut);
+    setPortOut(parStore->list.at(ix).portOut);
+    updatePortOut(parStore->list.at(ix).portOut);
+    updateLoop(parStore->list.at(ix).loopMode);
+
+    updateWaveForm(parStore->list.at(ix).waveForm);
+    midiWorker->setCurrentIndex(0);
+}
+
 void SeqWidget::copyParamsFrom(SeqWidget *fromWidget)
 {
     int tmp;
@@ -874,6 +935,7 @@ void SeqWidget::copyParamsFrom(SeqWidget *fromWidget)
     enableVelIn->setChecked(fromWidget->enableVelIn->isChecked());
     enableRestartByKbd->setChecked(fromWidget->enableRestartByKbd->isChecked());
     enableTrigByKbd->setChecked(fromWidget->enableTrigByKbd->isChecked());
+    enableTrigLegato->setChecked(fromWidget->enableTrigLegato->isChecked());
 
     tmp = fromWidget->chIn->currentIndex();
     chIn->setCurrentIndex(tmp);
@@ -909,7 +971,7 @@ void SeqWidget::copyParamsFrom(SeqWidget *fromWidget)
         midiWorker->muteMask.append(midiWorker->customWave.at(l1).muted);
     }
     tmp = fromWidget->getLoopMarker();
-    midiWorker->loopMarker = tmp;
+    midiWorker->setLoopMarker(tmp);
     screen->setLoopMarker(tmp);
     midiControl->setCcList(fromWidget->midiControl->ccList);
     muteOut->setChecked(true);
