@@ -4,7 +4,7 @@
  *
  * @section LICENSE
  *
- *      Copyright 2009, 2010, 2011 <qmidiarp-devel@lists.sourceforge.net>
+ *      Copyright 2009, 2010, 2011, 2012 <qmidiarp-devel@lists.sourceforge.net>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -41,10 +41,8 @@
 #include "seqscreen.h"
 #include "midicontrol.h"
 #include "managebox.h"
+#include "parstore.h"
 
-/*! @brief This array holds the currently available resolution values.
- */
-const int seqResValues[5] = {1, 2, 4, 8, 16};
 
 /*!
  * @brief GUI class associated with and controlling a MidiSeq worker
@@ -69,16 +67,6 @@ class SeqWidget : public QWidget
     bool modified;      /**< Is set to True if unsaved parameter modifications exist */
     bool lastMute;      /**< Contains the mute state of the last waveForm point modified by mouse click*/
 /*!
-* @brief This function allows ignoring one XML element in the XML stream
-* passed by the caller.
-*
-* It also advances the stream read-in. It is used to
-* ignore unknown elements for both-ways-compatibility
-*
-* @param xml reference to QXmlStreamReader containing the open XML stream
-*/
-    void skipXmlElement(QXmlStreamReader& xml);
-/*!
 * @brief This function populates the SeqWidget::waveForms list with
 * waveform names, currently only one.
 *
@@ -108,6 +96,7 @@ class SeqWidget : public QWidget
             bool mutedAdd = false, QWidget* parent=0);
     ~SeqWidget();
 
+    ParStore *parStore;
     MidiControl *midiControl;
     SeqScreen *screen;
     ManageBox *manageBox;
@@ -123,6 +112,7 @@ class SeqWidget : public QWidget
     QCheckBox *enableVelIn;
     QCheckBox *enableRestartByKbd;
     QCheckBox *enableTrigByKbd;
+    QCheckBox *enableTrigLegato;
     QCheckBox *dispVert[4];
     Slider *velocity, *transpose, *notelength;
     QAction *recordAction;
@@ -155,6 +145,20 @@ class SeqWidget : public QWidget
 */
     void writeData(QXmlStreamWriter& xml);
 /*!
+* @brief This function stores some module parameters in a parameter
+* list object
+*
+* @param Position index in the parameter list
+*/
+    void storeParams(int ix, bool empty = false);
+/*!
+* @brief This function restores some module parameters from the parameter
+* list object
+*
+* @param Position index in the parameter list
+*/
+    void restoreParams(int ix);
+/*!
 * @brief This function copies all Seq module GUI parameters from
 * fromWidget
 *
@@ -253,6 +257,7 @@ class SeqWidget : public QWidget
     void updateEnableNoteOff(bool on);
     void updateEnableRestartByKbd(bool on);
     void updateEnableTrigByKbd(bool on);
+    void updateTrigLegato(bool on);
     void setRecord(bool on);
     void setDispVert(int mode);
     void updateDispVert(int mode);
